import unittest
from circlator import external_progs

class TestExternalProgs(unittest.TestCase):
    
    def test_canu_version(self):
            '''Test canu version'''
            self.assertEqual('1.6', self.check_regex_version_extraction('canu', """
Canu 1.6
            """ ))
    
    def test_spades_version(self):
            '''Test spades version'''
            self.assertEqual('3.11.0', self.check_regex_version_extraction('spades', """
SPAdes v3.11.0
            """ ))
            self.assertEqual('3.7.1', self.check_regex_version_extraction('spades', """
SPAdes v3.7.1
            """ ))
            self.assertEqual('3.5.0', self.check_regex_version_extraction('spades', """
SPAdes genome assembler v.3.5.0
            """ ))
    
    def test_prodigal_version(self):
            '''Test prodigal version'''
            self.assertEqual('2.60', self.check_regex_version_extraction('prodigal', """

Prodigal V2.60: October, 2011

            """ ))
    
    def test_bwa_version(self):
        '''Test bwa version'''
        self.assertEqual('0.7.10', self.check_regex_version_extraction('bwa', """

Program: bwa (alignment via Burrows-Wheeler transformation)
Version: 0.7.10-r789
Contact: Heng Li <lh3@sanger.ac.uk>

        """ ))
        self.assertEqual('0.7.12', self.check_regex_version_extraction('bwa', """

Program: bwa (alignment via Burrows-Wheeler transformation)
Version: 0.7.12-r1039
Contact: Heng Li <lh3@sanger.ac.uk>
         """ ))

    def test_nucmer_version(self):    
        '''Test nucmer version'''
        self.assertEqual('3.1', self.check_regex_version_extraction('nucmer', """
nucmer
NUCmer (NUCleotide MUMmer) version 3.1
        """ ))
        self.assertEqual('4.0.0', self.check_regex_version_extraction('nucmer', """
4.0.0beta1
        """ ))
    
    def test_samtools_version(self):
        '''Test samtools version'''
        self.assertEqual('1.6', self.check_regex_version_extraction('samtools', """

Program: samtools (Tools for alignments in the SAM format)
Version: 1.6 (using htslib 1.6)
""" ))

    def test_samtools_original_version(self):
        '''Test samtools original version'''
        self.assertEqual('0.1.19', self.check_regex_version_extraction('samtools', """
Program: samtools (Tools for alignments in the SAM format)
Version: 0.1.19-44428cd

Usage:   samtools <command> [options]""" ))
         
    def check_regex_version_extraction(self, prog,  raw_version_output ):
        cmd, regex = external_progs.prog_to_version_cmd[prog]
        raw_output_lines = raw_version_output.splitlines()
        for line in raw_output_lines:
            hits = regex.search(line)
            if hits:
                return str(hits.group(1))
        return None
        


import sys
import openpyxl
import pyfastaq
import pymummer
import pysam
from circlator import external_progs, __version__
from circlator import __version__ as circlator_version


def get_all_versions(filehandle, raise_error=True, debug=False, assembler=None):
    if filehandle is not None:
        print('Circlator version:', circlator_version, file=filehandle)
        print('\nExternal dependencies:', file=filehandle)

    external_progs.check_all_progs(verbose=False, raise_error=raise_error, filehandle=filehandle, debug=debug, assembler=assembler)

    if filehandle is not None:
        print('\nPython version:', file=filehandle)
        print(sys.version, file=filehandle)
        print('\nPython dependencies:', file=filehandle)

    found_bad_module = False

    for module in ['openpyxl', 'pyfastaq', 'pymummer', 'pysam']:
        try:
            version = eval(module + '.__version__')
            path = eval(module + '.__file__')
        except:
            version = 'NOT_FOUND'
            path = 'NOT_FOUND'
            found_bad_module = True

        if filehandle is not None:
            print(module + '\t' + version + '\t' + path, file=filehandle)

    if raise_error and found_bad_module:
        print('Some dependencies not satisfied. Cannot continue. Try running: circlator progcheck', file=sys.stderr)
        sys.exit(1)




import re
import sys
from circlator import program, common
from circlator import __version__ as circlator_version
import shutil
import pyfastaq

class Error (Exception): pass

prog_to_env_var = {
    'samtools': 'CIRCLATOR_SAMTOOLS',
    'spades': 'CIRCLATOR_SPADES',
    'canu': 'CIRCLATOR_CANU',
}

prog_to_version_cmd = {
    'bwa': ('', re.compile(r'^Version: ([0-9\.]+)')),
    'nucmer': ('--version', re.compile(r'([0-9\.]+)')),
    'prodigal': ('-v', re.compile(r'^Prodigal V([0-9\.]+):')),
    'samtools': ('', re.compile(r'Version: (\d+\.\d+[\.\d]*)')),
    'spades': ('-v', re.compile(r'v.?([0-9][0-9\.]+)')),
    'canu': ('-version', re.compile(r'^Canu \D*([\d][\d\.]+)')),
}

min_versions = {
    'bwa': '0.7.12',
    'nucmer': '3.1',
    'prodigal': '2.6',
    'samtools': '0.1.19',
    'spades': '3.6.2', # this is the first version to support python3
    'canu': '0.0',
}


bad_versions = {
    'spades': '3.6.1'
}


prog_name_to_default = {
    'bwa': 'bwa',
    'nucmer': 'nucmer',
    'prodigal': 'prodigal',
    'spades': 'spades.py',
    'samtools': 'samtools',
    'canu': 'canu',
}

not_required = {'spades', 'canu'}

def handle_error(message, raise_error=True):
    if raise_error:
        raise Error(message + '\nCannot continue')
    else:
        print(message)


def make_and_check_prog(name, verbose=False, raise_error=True, filehandle=None, debug=False, required=False):
    p = program.Program(
        prog_name_to_default[name],
        prog_to_version_cmd[name][0],
        prog_to_version_cmd[name][1],
        environment_var=prog_to_env_var.get(name, None),
        debug=debug
    )

    if not p.in_path():
        if required:
            die = True
        else:
            die = raise_error and (name not in not_required)
        handle_error("WARNING: Didn't find " + name + " in path. Looked for:" + p.path, raise_error=die)
        return p

    version = p.version

    if version is None:
        handle_error('Found ' + name + " but couldn't get version.", raise_error=raise_error)
        return p

    if not p.version_at_least(min_versions[name]):
        handle_error('Version of ' + name + ' too low. I found ' + p.version + ', but must be at least ' + min_versions[name] + '. Found here:\n' + p.from_which, raise_error=raise_error)
        return p

    if name == 'spades' and p.version == bad_versions['spades']:
        handle_error('ERROR! SPAdes version ' + bad_versions['spades'] + ' is incompatible with Circlator. Please use SPAdes 3.7.1', raise_error=raise_error)
        return p

    if name == 'spades' and not p.version.startswith('3.7.'):
        print('WARNING: SPAdes version', p.version, 'is being used. It will work, but better results are usually obtained from Circlator using SPAdes version 3.7.1. Although 3.7.1 is not the latest version, we recommend it for Circlator.', file=sys.stderr)

    if verbose:
        print(name, p.version, p.from_which, sep='\t')

    if filehandle:
        print(name, p.version, p.from_which, sep='\t', file=filehandle)

    return p


def check_all_progs(verbose=False, raise_error=False, filehandle=None, debug=False, assembler=None):
    for prog in sorted(prog_name_to_default):
        if debug:
            print('__________ checking', prog, '____________', flush=True)
        make_and_check_prog(prog, verbose=verbose, raise_error=raise_error, filehandle=filehandle, debug=debug, required=prog==assembler)



import unittest
import filecmp
import os
import pymummer
from circlator import clean

modules_dir = os.path.dirname(os.path.abspath(clean.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestClean(unittest.TestCase):
    def test_get_contigs_to_keep(self):
        '''test _get_contigs_to_keep'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        self.assertEqual(cleaner._get_contigs_to_keep(None), set())
        test_file = os.path.join(data_dir, 'clean_test_contigs_to_keep')
        self.assertEqual(cleaner._get_contigs_to_keep(test_file), {'contig42', 'contig64738'})


    def test_remove_small_contigs(self):
        '''test _remove_small_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix', min_contig_length=2)
        infile = os.path.join(data_dir, 'clean_test_remove_small_contigs.in.fa')
        expected_out = os.path.join(data_dir, 'clean_test_remove_small_contigs.out.fa')
        expected_removed = {'contig1'}
        expected_all = {'contig1', 'contig2', 'contig3', 'contig4'}
        got_out = 'tmp.test_remove_small_contigs.out.fa'
        got_all, got_removed = cleaner._remove_small_contigs(infile, got_out, {'contig4'})
        self.assertTrue(filecmp.cmp(expected_out, got_out, shallow=False))
        self.assertEqual(expected_removed, got_removed)
        self.assertEqual(expected_all, got_all)
        os.unlink(got_out)


    def test_load_nucmer_hits(self):
        '''test _load_nucmer_hits'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        expected_lengths = {'contig1': 1000, 'contig2': 1002, 'contig3': 960, 'contig4': 540}
        hits = [
            '\t'.join(['1', '410', '1', '410', '410', '410', '100.00', '1000', '960', '1', '1', 'contig1', 'contig3']),
            '\t'.join(['481', '960', '481', '960', '480', '480', '100.00', '1002', '960', '1', '1', 'contig2', 'contig3']),
            '\t'.join(['481', '780', '241', '540', '300', '300', '100.00', '1002', '540', '1', '1', 'contig2', 'contig4']),
            '\t'.join(['1', '410', '1', '410', '410', '410', '100.00', '960', '1000', '1', '1', 'contig3', 'contig1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]

        expected_hits = {
            'contig3': [hits[0], hits[1]],
            'contig4': [hits[2]],
            'contig1': [hits[3]],
        }

        coords_file = os.path.join(data_dir, 'clean_test_load_nucmer_hits.coords')
        got_lengths, got_hits = cleaner._load_nucmer_hits(coords_file)
        self.assertEqual(expected_lengths, got_lengths)
        self.assertEqual(expected_hits, got_hits)


    def test_contains(self):
        '''test _contains'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        cleaner.contigs_to_keep = {'contig4'}
        hits = [
            '\t'.join(['1', '95', '1', '95', '95', '95', '100.00', '100', '100', '1', '1', 'contig1', 'contig2']),
            '\t'.join(['1', '95', '1', '95', '95', '95', '94.00', '100', '100', '1', '1', 'contig1', 'contig2']),
            '\t'.join(['2', '95', '1', '94', '94', '94', '100.00', '100', '100', '1', '1', 'contig1', 'contig2']),
            '\t'.join(['1', '95', '1', '95', '95', '95', '100.00', '100', '100', '1', '1', 'contig3', 'contig4']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]

        expected = [
            True,
            False,
            False,
            False,
        ]

        assert len(expected) == len(hits)

        for i in range(len(hits)):
            self.assertEqual(expected[i], cleaner._contains(hits[i]))


    def test_containing_contigs(self):
        '''test _containing_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        hits = [
            '\t'.join(['1', '95', '1', '95', '95', '95', '100.00', '100', '100', '1', '1', 'contig1', 'contig2']),
            '\t'.join(['1', '95', '1', '95', '95', '95', '94.00', '100', '100', '1', '1', 'contig3', 'contig2']),
            '\t'.join(['2', '95', '1', '94', '94', '94', '100.00', '100', '100', '1', '1', 'contig4', 'contig2']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]

        self.assertEqual(cleaner._containing_contigs(hits), {'contig1'})


    def test_get_containing_contigs(self):
        '''test _get_containing_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        hits = [
            '\t'.join(['1', '95', '1', '95', '95', '95', '100.00', '100', '100', '1', '1', 'contig1', 'contig2']),
            '\t'.join(['1', '95', '1', '95', '95', '95', '94.00', '100', '100', '1', '1', 'contig3', 'contig2']),
            '\t'.join(['2', '95', '1', '94', '94', '94', '100.00', '100', '100', '1', '1', 'contig4', 'contig2']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        dict_in = {'contig2': hits}
        expected = {'contig2': {'contig1'}}
        self.assertEqual(cleaner._get_containing_contigs(dict_in), expected)


    def test_get_all_containing(self):
        '''test _get_all_containing'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        dict_in = {
            'a': {'b'},
            'b': {'c'},
            'c': {'d'},
            'e': {'f'},
            'f': {'e', 'g'},
        }

        name_and_expected = [
            ('a', {'b','c','d'}),
            ('b', {'c', 'd'}),
            ('c', {'d'}),
            ('e', {'f', 'g'}),
            ('f', {'e', 'g'})
        ]

        for name, expected in name_and_expected:
            self.assertEqual(expected, cleaner._get_all_containing(dict_in, name))


    def test_expand_containing_using_transitivity(self):
        '''test _expand_containing_using_transitivity'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        dict_in = {
            'a': {'b'},
            'b': {'c'},
            'c': {'d'},
            'e': {'f', 'g'},
            'h': {'e'},
            'i': {'j'},
            'k': {'j'},
        }

        expected = {
            'a': {'b', 'c', 'd'},
            'b': {'c', 'd'},
            'c': {'d'},
            'e': {'f', 'g'},
            'h': {'e', 'f', 'g'},
            'i': {'j'},
            'k': {'j'},
        }

        got = cleaner._expand_containing_using_transitivity(dict_in)
        self.assertEqual(expected, got)
		
		

    def test_infinite_recursion(self):
        '''test _infinite_recursion'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        dict_in = {
            'a': {'b'},
            'b': {'c'},
            'c': {'a', 'b'},
        }

        expected = {'a': {'b', 'c'}, 'b': {'a', 'c'}, 'c': {'a', 'b'}}

        got = cleaner._expand_containing_using_transitivity(dict_in)
        self.assertEqual(expected, got)

    def test_collapse_list_of_sets(self):
        '''test _collapse_list_of_sets'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        tests = [
            ([{1}], [{1}]),
            ([{1,2}, {1}], [{1,2}]),
            ([{1,2,3}, {4,5,6}], [{1,2,3}, {4,5,6}]),
            ([{1,2,3}, {4,5,6}, {4,5,6}], [{1,2,3}, {4,5,6}]),
            ([{1,2,3}, {4,5,6}, {1,5,6}], [{1,2,3,4,5,6}])
        ]

        for list_in, expected in tests:
            self.assertEqual(cleaner._collapse_list_of_sets(list_in), expected)


    def test_get_identical_contigs(self):
        '''test _get_identical_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        tests = [
            ({1: {2}, 2: {1}}, [{1,2}]),
            ({1: {2}, 2: {1}, 3: {1}}, [{1,2}]),
            ({1: {2,3}, 2: {1}, 3: {1}}, [{1,2,3}]),
            ({1: {2,3}, 2: {1}, 3: {1}, 4: {5}, 5: {6}}, [{1,2,3}]),
            ({1: {2,3}, 2: {1}, 3: {1}, 4: {5}, 5: {6}, 6: {4}}, [{1,2,3}])
        ]

        for dict_in, expected in tests:
            self.assertEqual(cleaner._get_identical_contigs(dict_in), expected)


    def test_longest_contig(self):
        '''test _longest_contig'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        lengths = {'a': 42, 'b': 1, 'c': 123}
        names = {'a', 'b', 'c'}
        self.assertEqual('c', cleaner._longest_contig(names, lengths))


    def test_remove_identical_contigs(self):
        '''test _remove_identical_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        dict_in = {
            'a': {'b'},
            'c': {'d'},
            'd': {'c'},
            'e': {'f', 'g'},
            'f': {'e'},
            'h': {'i', 'j'},
            'i': {'j', 'h'},
            'j': {'i', 'h'},
            'k': {'l'},
            'l': {'m'},
            'n': {'m'},
            'p': {'q', 'r', 's'},
            'q': {'r', 's'},
            'r': {'s'},
        }

        lengths = {
            'a': 10,
            'b': 20,
            'c': 40,
            'd': 42,
            'e': 10,
            'f': 11,
            'g': 10,
            'h': 20,
            'i': 21,
            'j': 20,
            'k': 20,
            'l': 20,
            'm': 20,
            'n': 20,
            'p': 20,
            'q': 20,
            'r': 20,
        }

        expected_contained = {
            'a': {'b'},
            'f': {'g'},
            'k': {'l'},
            'l': {'m'},
            'n': {'m'},
            'p': {'q', 'r', 's'},
            'q': {'r', 's'},
            'r': {'s'},
        }

        expected_replaced = {
            'c': 'd',
            'e': 'f',
            'h': 'i',
            'j': 'i'
        }

        got_contained, got_replaced = cleaner._remove_identical_contigs(dict_in, lengths)
        self.assertEqual(expected_contained, got_contained)
        self.assertEqual(expected_replaced, got_replaced)


    def test_clean_contigs(self):
        '''test _clean_contigs'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        infile = os.path.join(data_dir, 'clean_test_clean_contigs.in.fa')
        expected_outfile = os.path.join(data_dir, 'clean_test_clean_contigs.out.fa')
        outfile = 'tmp.test_clean_contigs.out.fa'
        containing_contigs = {'contig1'}
        replaced_contigs = {'contig2'}
        cleaner._clean_contigs(infile, outfile, containing_contigs, replaced_contigs)
        self.assertTrue(filecmp.cmp(expected_outfile, outfile, shallow=False))
        os.unlink(outfile)


    def test_write_log(self):
        '''test _write_log'''
        cleaner = clean.Cleaner('infile', 'outprefix')
        cleaner.contigs_to_keep = {'2'}
        outfile = 'tmp.test_clean_write_log'
        all_contigs = {'1', '2', '3', '4', '5', '6', '7', '8', '9'}
        small_removed = '3'
        containing_contigs = {'4': {'5', '6'}}
        replaced_contigs = {'7': '8'}
        cleaner._write_log(outfile, '[clean]', all_contigs, small_removed, containing_contigs, replaced_contigs)
        expected = os.path.join(data_dir, 'clean_test_write_log.log')
        self.assertTrue(filecmp.cmp(expected, outfile, shallow=False))
        os.unlink(outfile)



import os
import pysam
import pyfastaq
from circlator import common, mapping

class Error (Exception): pass


class BamFilter:
    def __init__(
             self,
             bam,
             outprefix,
             fastq_out=False,
             length_cutoff=100000,
             min_read_length=250,
             contigs_to_use=None,
             discard_unmapped=False,
             log_prefix='[bamfilter]',
             verbose=False,
             split_all_reads=False,
    ):
        self.bam = os.path.abspath(bam)
        if not os.path.exists(self.bam):
            raise Error('File not found:' + self.bam)

        self.fastq_out = fastq_out
        self.length_cutoff = length_cutoff
        self.reads_outfile = os.path.abspath(outprefix + ('.fastq' if self.fastq_out else '.fasta'))
        self.log = os.path.abspath(outprefix + '.log')
        self.log_prefix = log_prefix
        self.contigs_to_use = self._get_contigs_to_use(contigs_to_use)
        self.discard_unmapped = discard_unmapped
        self.min_read_length = min_read_length
        self.verbose = verbose
        self.split_all_reads = split_all_reads



    def _get_ref_lengths(self):
        '''Gets the length of each reference sequence from the header of the bam. Returns dict name => length'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        return dict(zip(sam_reader.references, sam_reader.lengths))


    def _get_contigs_to_use(self, contigs_to_use):
        '''If contigs_to_use is a set, returns that set. If it's None, returns an empty set.
        Otherwise, assumes it's a file name, and gets names from the file'''
        if type(contigs_to_use) == set:
            return contigs_to_use
        elif contigs_to_use is None:
            return set()
        else:
            f = pyfastaq.utils.open_file_read(contigs_to_use)
            contigs_to_use = set([line.rstrip() for line in f])
            pyfastaq.utils.close(f)
            return contigs_to_use


    def _check_contigs_to_use(self, ref_dict):
        '''Checks that the set of contigs to use are all in the reference
        fasta lengths dict made by self._get_ref_lengths()'''
        if self.contigs_to_use is None:
            return True

        for contig in self.contigs_to_use:
            if contig not in ref_dict:
                raise Error('Requested to use contig "' + contig + '", but not found in input BAM file "' + self.bam + '"')

        return True


    def _all_reads_from_contig(self, contig, fout):
        '''Gets all reads from contig called "contig" and writes to fout'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        for read in sam_reader.fetch(contig):
            print(mapping.aligned_read_to_read(read, ignore_quality=not self.fastq_out), file=fout)


    def _get_all_unmapped_reads(self, fout):
        '''Writes all unmapped reads to fout'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        for read in sam_reader.fetch(until_eof=True):
            if read.is_unmapped:
                print(mapping.aligned_read_to_read(read, ignore_quality=not self.fastq_out), file=fout)


    def _break_reads(self, contig, position, fout, min_read_length=250):
        '''Get all reads from contig, but breaks them all at given position (0-based) in the reference. Writes to fout. Currently pproximate where it breaks (ignores indels in the alignment)'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        for read in sam_reader.fetch(contig):
            seqs = []
            if read.pos < position < read.reference_end - 1:
                split_point = position - read.pos
                if split_point - 1 >= min_read_length:
                    sequence = mapping.aligned_read_to_read(read, revcomp=False, ignore_quality=not self.fastq_out).subseq(0, split_point)
                    sequence.id += '.left'
                    seqs.append(sequence)
                if read.query_length - split_point >= min_read_length:
                    sequence = mapping.aligned_read_to_read(read, revcomp=False, ignore_quality=not self.fastq_out).subseq(split_point, read.query_length)
                    sequence.id += '.right'
                    seqs.append(sequence)
            else:
                seqs.append(mapping.aligned_read_to_read(read, revcomp=False, ignore_quality=not self.fastq_out))

            for seq in seqs:
                if read.is_reverse:
                    seq.revcomp()
                print(seq, file=fout)


    def _exclude_region(self, contig, start, end, fout):
        '''Writes reads not mapping to the given region of contig, start and end as per python convention'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        exclude_interval = pyfastaq.intervals.Interval(start, end - 1)
        for read in sam_reader.fetch(contig):
            read_interval = pyfastaq.intervals.Interval(read.pos, read.reference_end - 1)
            if not read_interval.intersects(exclude_interval):
                print(mapping.aligned_read_to_read(read, ignore_quality=not self.fastq_out), file=fout)


    def _get_region(self, contig, start, end, fout, min_length=250):
        '''Writes reads mapping to given region of contig, trimming part of read not in the region'''
        sam_reader = pysam.Samfile(self.bam, "rb")
        trimming_end = (start == 0)
        for read in sam_reader.fetch(contig, start, end):
            read_interval = pyfastaq.intervals.Interval(read.pos, read.reference_end - 1)
            seq = mapping.aligned_read_to_read(read, ignore_quality=not self.fastq_out, revcomp=False)

            if trimming_end:
                bases_off_start = 0
                bases_off_end = max(0, read.reference_end - 1 - end)
                #seq.seq = seq.seq[:read.query_alignment_end - bases_off_end]
                seq = seq.subseq(0, read.query_alignment_end - bases_off_end)
            else:
                bases_off_start = max(0, start - read.pos + 1)
                #seq.seq = seq.seq[bases_off_start  + read.query_alignment_start:]
                seq = seq.subseq(bases_off_start  + read.query_alignment_start, len(seq))

            if read.is_reverse:
                seq.revcomp()

            if len(seq) >= min_length:
                print(seq, file=fout)


    def run(self):
        ref_lengths = self._get_ref_lengths()
        assert len(ref_lengths) > 0
        f_log = pyfastaq.utils.open_file_write(self.log)
        f_fa = pyfastaq.utils.open_file_write(self.reads_outfile)
        print(self.log_prefix, '#contig', 'length', 'reads_kept', sep='\t', file=f_log)
        if self.verbose:
            print('Getting reads from BAM file', self.bam, flush=True)

        for contig in sorted(ref_lengths):
            if len(self.contigs_to_use) > 0 and contig not in self.contigs_to_use:
                print(self.log_prefix, contig, ref_lengths[contig], 'skipping', sep='\t', file=f_log)
                if self.verbose:
                    print('Skipping contig', contig, flush=True)
                continue

            if ref_lengths[contig] <= self.length_cutoff:
                #NEW version: even for small contigs, split them in two halves, and split the reads mapping through the middle point of the contig in two, so that each piece maps to only one half.
                if self.split_all_reads:
                    end_bases_keep = int(0.5 * ref_lengths[contig])
                    start = end_bases_keep - 1
                    end = end_bases_keep #max(end_bases_keep - 1, ref_lengths[contig] - end_bases_keep)
                    self._get_region(contig, 0, start, f_fa, min_length=self.min_read_length)
                    self._get_region(contig, end, ref_lengths[contig], f_fa, min_length=self.min_read_length)
                    coords_string = '1-' + str(start + 1) +  ';' + str(end + 1) + '-' + str(ref_lengths[contig])
                    print(
                        self.log_prefix,
                        contig,
                        ref_lengths[contig],
                        coords_string,
                        sep='\t',
                        file=f_log
                    )
                    if self.verbose:
                        print('Getting all reads that map to ends of contig ', contig, '. Coords: ', coords_string, sep='', flush=True)
                else:
                    #OLD version, do not split reads so that SPAdes attempts to circulrize.
                    self._all_reads_from_contig(contig, f_fa)
                    print(self.log_prefix, contig, ref_lengths[contig], 'all', sep='\t', file=f_log)
                    if self.verbose:
                        print('Getting all reads that map to contig', contig, flush=True)
                    
            else:
                end_bases_keep = int(0.5 * self.length_cutoff)
                start = end_bases_keep - 1
                end = max(end_bases_keep - 1, ref_lengths[contig] - end_bases_keep)
                self._get_region(contig, 0, start, f_fa, min_length=self.min_read_length)
                self._get_region(contig, end, ref_lengths[contig], f_fa, min_length=self.min_read_length)
                coords_string = '1-' + str(start + 1) +  ';' + str(end + 1) + '-' + str(ref_lengths[contig])
                print(
                    self.log_prefix,
                    contig,
                    ref_lengths[contig],
                    coords_string,
                    sep='\t',
                    file=f_log
                )
                if self.verbose:
                    print('Getting all reads that map to ends of contig ', contig, '. Coords: ', coords_string, sep='', flush=True)

        if not self.discard_unmapped:
            if self.verbose:
                print('Getting all unmapped reads', sep='', flush=True)
            self._get_all_unmapped_reads(f_fa)

        pyfastaq.utils.close(f_fa)
        pyfastaq.utils.close(f_log)

        if self.verbose:
            print('Finished getting reads.')
            print('Log file:', self.log)
            print('Reads file:', self.reads_outfile, flush=True)



import unittest
import copy
import filecmp
import os
import pysam
import pyfastaq
from circlator import mapping

modules_dir = os.path.dirname(os.path.abspath(mapping.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestMapping(unittest.TestCase):
    def test_bwa_index(self):
        '''test bwa_index'''
        ref = os.path.join(data_dir, 'mapping_test_bwa_index.fa')
        outprefix = 'tmp.mapping_test.bwa_index'
        mapping.bwa_index(ref, outprefix)
        expected_files = [outprefix + '.' + x for x in ['amb', 'ann', 'bwt', 'pac', 'sa']]

        for filename in expected_files:
            self.assertTrue(os.path.exists(filename))
            os.unlink(filename)


    def test_bwa_index_clean(self):
        '''test bwa_index_clean'''
        files_prefix = 'tmp.test_bwa_index_clean'
        test_files = [files_prefix + '.' + x for x in ['amb', 'ann', 'bwt', 'pac', 'sa']]
        for filename in test_files:
            with open(filename, 'w') as f:
                pass

            self.assertTrue(os.path.exists(filename))

        mapping.bwa_index_clean(files_prefix)

        for filename in test_files:
            self.assertFalse(os.path.exists(filename))


    def test_bwa_mem(self):
        '''test bwa_mem'''
        ref = os.path.join(data_dir, 'mapping_test_bwa_mem.ref.fa')
        reads = os.path.join(data_dir, 'mapping_test_bwa_mem.reads.fq')
        outfile = 'tmp.mapping_test_bwa_mem.bam'
        mapping.bwa_mem(ref, reads, outfile)
        self.assertTrue(os.path.exists(outfile))
        self.assertTrue(os.path.exists(outfile + '.bai'))

        expected_reads = [
            '1:2:49:172',
            '1:1:113:235',
            '1:3:205:315',
            '2:5:27:142',
            '2:4:101:224',
            '2:6:214:330',
        ]

        sam_reader = pysam.Samfile(outfile, "rb")
        got_reads = []

        for read in sam_reader.fetch():
            got_reads.append(read.qname)
            self.assertFalse(read.is_unmapped)

        self.assertEqual(expected_reads, got_reads)
        os.unlink(outfile)
        os.unlink(outfile + '.bai')


    def test_aligned_read_to_read(self):
        '''test aligned_read_to_read'''
        infile = os.path.join(data_dir, 'mapping_test_aligned_read_to_read.bam')
        sam_reader = pysam.Samfile(infile, "rb")
        aln1, aln2 = [x for x in sam_reader.fetch()]
        read1_fq = pyfastaq.sequences.Fastq('read1', 'TGTGTAACACTCCACCTCTGGTTCCCAGAGTTCGGTATCCGGCCGATACTTGAGGATAGC', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIGHFEDCBA')
        read1_fa = pyfastaq.sequences.Fasta('read1', 'TGTGTAACACTCCACCTCTGGTTCCCAGAGTTCGGTATCCGGCCGATACTTGAGGATAGC')
        self.assertEqual(read1_fq, mapping.aligned_read_to_read(aln1))
        self.assertEqual(read1_fq, mapping.aligned_read_to_read(aln1, revcomp=False))
        self.assertEqual(read1_fa, mapping.aligned_read_to_read(aln1, ignore_quality=True))

        read2 = pyfastaq.sequences.Fastq('read2', 'GATCGTCACGAAAGAACCAAGCCGGATCGTGGGAGGGGTACAACTCAGGTGAATTAACGT', 'HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHGFEDC')
        read2_rev = copy.copy(read2)
        read2_rev.revcomp()
        self.assertEqual(read2, mapping.aligned_read_to_read(aln2))
        self.assertEqual(read2_rev, mapping.aligned_read_to_read(aln2, revcomp=False))




import unittest
import filecmp
import os
import pyfastaq
from circlator import dnaa

modules_dir = os.path.dirname(os.path.abspath(dnaa.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestDnaa(unittest.TestCase):
    def test_header_to_genus_species(self):
        '''test _header_to_genus_species'''
        downloader = dnaa.UniprotDownloader()
        good_header = '>sp|P03004|DNAA_ECOLI Chromosomal replication initiator protein DnaA OS=Escherichia coli (strain K12) GN=dnaA PE=1 SV=2'
        bad_header = '>sp|P03004|DNAA_ECOLI Chromosomal replication initiator protein DnaA coli (strain K12) GN=dnaA PE=1 SV=2',

        self.assertEqual(downloader._header_to_genus_species(good_header), ('Escherichia', 'coli'))
        self.assertEqual(downloader._header_to_genus_species(bad_header), None)


    def test_get_uniprot_url(self):
        '''test _get_uniprot_url'''
        tests = [
            ('dnaa', 'http://www.uniprot.org/uniprot/?sort=score&desc=&compress=no&query=dnaa&force=no&format=fasta'),
            ('spam eggs', 'http://www.uniprot.org/uniprot/?sort=score&desc=&compress=no&query=spam+eggs&force=no&format=fasta'),
            ('  blue yellow   green  ', 'http://www.uniprot.org/uniprot/?sort=score&desc=&compress=no&query=blue+yellow+green&force=no&format=fasta'),
        ]

        for search_term, expected in tests:
            downloader = dnaa.UniprotDownloader(uniprot_search=search_term)
            self.assertEqual(downloader._get_uniprot_url(), expected)


    def test_header_matches_regex(self):
        '''test _header_matches_regex'''
        tests = [
            ('dnaa', 'foo dnaa spam eggs', True, True),
            ('Dnaa', 'foo dnaa spam eggs', True, True),
            ('Dnaa', 'foo dnaa spam eggs', False, False),
            ('dnaa', 'foo DNAA spam eggs', True, True),
            ('dnaa', 'foo spam eggs', True, False),
            ('SPAM', 'foo spam eggs', True, True),
        ]

        for regex, sequence_name, ignorecase, expected in tests:
            downloader = dnaa.UniprotDownloader(header_regex=regex, header_regex_ignorecase=ignorecase)
            seq = pyfastaq.sequences.Fasta(sequence_name, 'ACGT')
            self.assertEqual(downloader._header_matches_regex(seq), expected)


    def test_check_sequence(self):
        '''test _check_sequence'''
        downloader = dnaa.UniprotDownloader(min_gene_length=3, max_gene_length=6)
        seen = set()
        tests = [
            (pyfastaq.sequences.Fasta('dnaa OS=genus1 species1 foo', 'MABC'), (True, None)),
            (pyfastaq.sequences.Fasta('Dnaa OS=genus1 species2 foo', 'MABC'), (True, None)),
            (pyfastaq.sequences.Fasta('DnaA OS=genus2 species1 foo', 'MABC'), (True, None)),
            (pyfastaq.sequences.Fasta('dnaA OS=genus2 species2 foo', 'MABC'), (True, None)),
            (pyfastaq.sequences.Fasta('DNAA OS=genus2 species3 foo', 'MABC'), (True, None)),
            (pyfastaq.sequences.Fasta('dnaa OS=genus3 species1 foo', 'MA'), (False, 'Too long or short')),
            (pyfastaq.sequences.Fasta('dnaa OS=genus3 species2 foo', 'MABCDEF'), (False, 'Too long or short')),
            (pyfastaq.sequences.Fasta('dnaa OS=genus3 species3 foo', 'XABC'), (False, 'Does not start with M')),
            (pyfastaq.sequences.Fasta('x OS=genus3 species4 foo', 'MABC'), (False, 'No match to regex in name')),
            (pyfastaq.sequences.Fasta('dnaa foo', 'MABC'), (False, 'Error getting genus species')),
            (pyfastaq.sequences.Fasta('dnaa OS=genus1 species1 foo', 'MABC'), (False, "Duplicate genus species ('genus1', 'species1')")),
        ]

        for fa, expected in tests:
            self.assertEqual(downloader._check_sequence(fa, seen), expected)


    def test_append_stop_to_seq(self):
        '''test _append_stop_to_seq'''
        downloader = dnaa.UniprotDownloader()
        fa = pyfastaq.sequences.Fasta('x', 'A')
        downloader._append_stop_to_seq(fa)
        self.assertEqual(fa.seq, 'A*')
        downloader._append_stop_to_seq(fa)
        self.assertEqual(fa.seq, 'A*')


    def test_reverse_translate(self):
        '''test _reverse_translate'''
        downloader = dnaa.UniprotDownloader()
        aa = pyfastaq.sequences.Fasta('x', 'W')
        nucl = pyfastaq.sequences.Fasta('x', 'TGG')
        downloader._reverse_translate(aa)
        self.assertEqual(aa, nucl)



import argparse
import os
import shutil
import sys
import subprocess
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Run Circlator on a small test dataset',
        usage = 'ariba test [options] <outdir>')
    parser.add_argument('--threads', type=int, help='Number of threads [%(default)s]', default=1, metavar='INT')
    parser.add_argument('outdir', help='Name of output directory')
    options = parser.parse_args()

    print('Running Circlator on test data...')

    try:
        os.mkdir(options.outdir)
        os.chdir(options.outdir)
    except:
        print('Error making output directory "', options.outdir, '". Cannot continue.', sep='', file=sys.stderr)
        sys.exit(1)

    print('Made output directory. Copying test data files into it:')

    modules_dir = os.path.dirname(os.path.abspath(circlator.__file__))
    test_data_dir = os.path.join(modules_dir, 'data')

    for filename in ['test_contigs.fa', 'test_reads.fq.gz']:
        shutil.copy(os.path.join(test_data_dir, filename), filename)
        print('    copied', filename)


    cmd = ' '.join([
        sys.argv[0],
        'all',
        '--threads', str(options.threads),
        '--verbose',
        '--assemble_spades_use_first',
        'test_contigs.fa',
        'test_reads.fq.gz',
        'OUT',
    ])

    print('\nRunning Circlator with:', cmd, '', sep='\n')

    return_code = subprocess.call(cmd, shell=True)

    if return_code != 0:
        print('\nSomething went wrong. See above for error message(s). Return code was', return_code)
        sys.exit(1)

    print('-' * 79)
    print('Finished run on test data OK')



import unittest
import filecmp
import os
import shutil
import pyfastaq
from circlator import minimus2

modules_dir = os.path.dirname(os.path.abspath(minimus2.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

# It's likely that some people will not want to run the minimus2 pipeline at
# all, and just use the circlator pipeline. So we check if amos is installed
# and only run the tests if it is installed.
try:
    merger = minimus2.Merger(
        os.path.join(data_dir, 'minimus2_test.dummy_input_assembly.fa'),
        'test.minimus2.out'
    )
    can_run_tests = True
except:
    can_run_tests = False


@unittest.skipUnless(can_run_tests, 'Amos is not installed')
class TestMinimus2(unittest.TestCase):
    def setUp(self):
        self.merger = minimus2.Merger(
            os.path.join(data_dir, 'minimus2_test.dummy_input_assembly.fa'),
            'test.minimus2.out'
        )


    def test_write_split_contig(self):
        '''test _write_split_contig'''
        contig = pyfastaq.sequences.Fasta('seq', 'ACGT')
        tmp_file = 'tmp.minimus2_test.write_split_contig.out'
        self.merger._write_split_contig(contig, tmp_file)
        expected = os.path.join(data_dir, 'minimus2_test.write_split_contig.fa')
        self.assertTrue(filecmp.cmp(tmp_file, expected, shallow=False))
        os.unlink(tmp_file)


    def test_run_minimus2(self):
        '''test _run_minimus2'''
        input_assembly = os.path.join(data_dir, 'minimus2_test_run_minimus2.in.fa')
        outdir = 'tmp.test_run_minimus2'
        os.mkdir(outdir)
        self.merger._run_minimus2(input_assembly, outdir)
        self.assertTrue(os.path.exists(os.path.join(outdir, 'minimus2.fasta')))
        shutil.rmtree(outdir)


    def test_run_minimus2_on_one_contig_will_not_work(self):
        '''test_run_minimus2_on_one_contig that won't circularise'''
        contig = pyfastaq.sequences.Fasta('seq', 'ACGT')
        new_contig, circularised = self.merger._run_minimus2_on_one_contig(contig)
        self.assertEqual(contig, new_contig)
        self.assertFalse(circularised)


    def test_run_minimus2_on_one_contig_will_work(self):
        '''test_run_minimus2_on_one_contig that will circularise'''
        contigs = {}
        contigs_file = os.path.join(data_dir, 'minimus2_test_run_minimus2_on_one_contig.in.fa')
        pyfastaq.tasks.file_to_dict(contigs_file, contigs)
        assert len(contigs) == 1
        contig = contigs['contig1']
        new_contig, circularised = self.merger._run_minimus2_on_one_contig(contig)
        self.assertTrue(circularised)
        expected_contigs = {}
        expected_contigs_file = os.path.join(data_dir, 'minimus2_test_run_minimus2_on_one_contig.expected.fa')
        pyfastaq.tasks.file_to_dict(expected_contigs_file, expected_contigs)
        assert len(expected_contigs) == 1
        expected_contig = expected_contigs['contig1.circularised']
        self.assertEqual(new_contig, expected_contig)


    def test_run_minimus2_on_each_contig(self):
        '''test _run_minimus2_on_each_contig'''
        input_contigs = os.path.join(data_dir, 'minimus2_test_run_minimus2_on_each_contig.in.fa')
        expected_contigs = os.path.join(data_dir, 'minimus2_test_run_minimus2_on_each_contig.expected.fa')
        tmpfile = 'tmp.test.run_minimus2_on_each_contig.out.fa'
        self.merger._run_minimus2_on_each_contig(input_contigs, tmpfile)
        self.assertTrue(filecmp.cmp(tmpfile, expected_contigs, shallow=False))
        os.unlink(tmpfile)


    def test_run_initial_merge(self):
        '''test _run_initial_merge'''
        input_contigs = os.path.join(data_dir, 'minimus2_test_run_initial_merge.in.fa')
        expected_contigs = os.path.join(data_dir, 'minimus2_test_run_initial_merge.expected.fa')
        tmpdir = 'tmp.test.run_initial_merge.out'
        got_file = self.merger._run_initial_merge(input_contigs, tmpdir)
        self.assertTrue(filecmp.cmp(expected_contigs, got_file, shallow=False))
        os.unlink(got_file)
        shutil.rmtree(tmpdir)




import unittest
import filecmp
import os
import shutil
import pyfastaq
from circlator import assemble

modules_dir = os.path.dirname(os.path.abspath(assemble.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestAssemble(unittest.TestCase):
    def setUp(self):
        self.tmp_assemble_dir = 'tmp.assemble_test'
        self.assembler = assemble.Assembler(
            os.path.join(data_dir, 'assemble_test.dummy_reads.fa'),
            self.tmp_assemble_dir
        )


    def test_build_spades_kmers(self):
        '''test _build_spades_kmers'''
        got = self.assembler._build_spades_kmers('127')
        self.assertEqual([127], got)

        got = self.assembler._build_spades_kmers('127,101,111')
        self.assertEqual([127, 101, 111], got)

        got = self.assembler._build_spades_kmers([55, 131, 75])
        self.assertEqual([55, 131, 75], got)

        with self.assertRaises(assemble.Error):
            self.assembler._build_spades_kmers(42)

        with self.assertRaises(assemble.Error):
            self.assembler._build_spades_kmers('shrubbery')

        with self.assertRaises(assemble.Error):
            self.assembler._build_spades_kmers('41,spam')


    def test_make_spades_command(self):
        '''test _make_spades_command'''
        cmd_start = ' '.join([
            self.assembler.spades.exe(),
            '-s', os.path.join(data_dir, 'assemble_test.dummy_reads.fa'),
        ])

        self.assertEqual(cmd_start + ' -o out -t 1 -k 41 --careful --only-assembler', self.assembler._make_spades_command(41, 'out'))
        self.assertEqual(cmd_start + ' -o out -t 1 -k 43 --careful --only-assembler', self.assembler._make_spades_command(43, 'out'))
        self.assertEqual(cmd_start + ' -o out2 -t 1 -k 41 --careful --only-assembler', self.assembler._make_spades_command(41, 'out2'))

        self.assembler.careful = False
        self.assertEqual(cmd_start + ' -o out -t 1 -k 41 --only-assembler', self.assembler._make_spades_command(41, 'out'))
        self.assembler.careful = True

        self.assembler.only_assembler = False
        self.assertEqual(cmd_start + ' -o out -t 1 -k 41 --careful', self.assembler._make_spades_command(41, 'out'))
        self.assembler.only_assembler = True

        self.assembler.threads = 2
        self.assertEqual(cmd_start + ' -o out -t 2 -k 41 --careful --only-assembler', self.assembler._make_spades_command(41, 'out'))


    def test_make_canu_command(self):
        '''test _make_canu_command'''
        tmp_assemble_dir = 'tmp.assemble_test'
        assembler = assemble.Assembler(
            os.path.join(data_dir, 'assemble_test.dummy_reads.fa'),
            tmp_assemble_dir,
            assembler='canu'
        )

        cmd_start = ' '.join([
            assembler.canu.exe(),
            '-useGrid=false',
            'gnuplotTested=true',
            '-assemble',
            'genomeSize=0.1m',
        ])

        reads = os.path.join(data_dir, 'assemble_test.dummy_reads.fa')
        self.assertEqual(cmd_start + ' -d out -p outname -pacbio-corrected ' + reads, assembler._make_canu_command('out', 'outname'))
        self.assertEqual(cmd_start + ' -d out2 -p outname2 -pacbio-corrected ' + reads, assembler._make_canu_command('out2', 'outname2'))
        assembler.data_type = 'pacbio-raw'
        self.assertEqual(cmd_start + ' -d out -p outname -pacbio-raw ' + reads, assembler._make_canu_command('out', 'outname'))


    def test_rename_canu_contigs(self):
        '''test _rename_canu_contigs'''
        infile = os.path.join(data_dir, 'assemble_test_rename_canu_contigs.in.fa')
        tmpfile = 'tmp.assemble_test_rename_canu_contigs.out.fa'
        expected = os.path.join(data_dir, 'assemble_test_rename_canu_contigs.expect.fa')
        assemble.Assembler._rename_canu_contigs(infile, tmpfile)
        self.assertTrue(filecmp.cmp(expected, tmpfile, shallow=False))
        os.unlink(tmpfile)



import argparse
import os
import sys
import pyfastaq
import circlator

class Error (Exception): pass


def run():
    parser = argparse.ArgumentParser(
        description = 'Runs minimus2 circularisation pipeline, see https://github.com/PacificBiosciences/Bioinformatics-Training/wiki/Circularizing-and-trimming ... this script is a modified version of that protocol. It first runs minimus2 on the input contigs (unless --no_pre_merge is used). Then it tries to circularise each contig one at a time, by breaking it in the middle and using the two pieces as input to minimus2. If minimus2 outputs one contig, then that new one is assumed to be circularised and is kept, otherwise the original contig is kept.',
        usage = 'circlator minimus2 [options] <assembly.fasta> <output prefix>')
    parser.add_argument('--no_pre_merge', action='store_true', help='Do not do initial minimus2 run before trying to circularise each contig')
    parser.add_argument('assembly', help='Name of original assembly', metavar='assembly.fasta')
    parser.add_argument('outprefix', help='Prefix of output files', metavar='output prefix')
    options = parser.parse_args()

    m = circlator.minimus2.Merger(
        options.assembly,
        options.outprefix,
        merge = not options.no_pre_merge
    )

    m.run()



import argparse
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Merge original and new assembly',
        usage = 'circlator merge [options] <original.fasta> <new.fasta> <outprefix>'
    )

    parser.add_argument('--diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    parser.add_argument('--min_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    parser.add_argument('--min_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    parser.add_argument('--min_length_merge', type=int, help='Minimum length of nucmer hit to use when merging [%(default)s]', metavar='INT', default=4000)
    parser.add_argument('--breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)
    parser.add_argument('--min_spades_circ_pc', type=float, help='Min percent of contigs needed to be covered by nucmer hits to spades circular contigs [%(default)s]', metavar='FLOAT', default=95)
    parser.add_argument('--assemble_not_careful', action='store_true', help='Do not use the --careful option with SPAdes (used by default)')
    parser.add_argument('--assemble_not_only_assembler', action='store_true', help='Do not use the --assemble-only option with SPAdes (used by default)')
    parser.add_argument('--spades_k', help='Comma separated list of kmers to use when running SPAdes. Max kmer is 127 and each kmer should be an odd integer [%(default)s]', default='127,117,107,97,87,77', metavar='k1,k2,k3,...')
    parser.add_argument('--spades_use_first', action='store_true', help='Use the first successful SPAdes assembly. Default is to try all kmers and use the assembly with the largest N50')
    parser.add_argument('--assembler', choices=circlator.common.allowed_assemblers, help='Assembler to use for reassemblies [%(default)s]', default='spades')
    parser.add_argument('--data_type', choices=circlator.common.allowed_data_types, help='String representing one of the 4 type of data analysed (only used for Canu) [%(default)s]', default='pacbio-corrected')
    parser.add_argument('--b2r_length_cutoff', type=int, help='All reads mapped to contigs shorter than this will be kept [%(default)s]', default=100000, metavar='INT')
    parser.add_argument('--b2r_split_all_reads', action='store_true', help='By default, reads mapped to shorter contigs are left unchanged. This option splits them into two, broken at the middle of the contig to try to force circularization. May help if the assembler does not detect circular contigs (eg canu)')
    parser.add_argument('--ref_end', type=int, help='max distance allowed between nucmer hit and end of input assembly contig [%(default)s]', metavar='INT', default=15000)
    parser.add_argument('--reassemble_end', type=int, help='max distance allowed between nucmer hit and end of reassembly contig [%(default)s]', metavar='INT', default=1000)
    parser.add_argument('--threads', type=int, help='Number of threads for remapping/assembly (only applies if --reads is used) [%(default)s]', default=1, metavar='INT')
    parser.add_argument('--reads', help='FASTA file of corrected reads that made the new assembly. Using this triggers iterative contig pair merging', metavar='FILENAME')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('original_assembly', help='Name of original assembly', metavar='original.fasta')
    parser.add_argument('new_assembly', help='Name of new assembly', metavar='new.fasta')
    parser.add_argument('outprefix', help='Prefix of output files')
    options = parser.parse_args()

    m = circlator.merge.Merger(
        options.original_assembly,
        options.new_assembly,
        options.outprefix,
        nucmer_diagdiff=options.diagdiff,
        nucmer_min_id=options.min_id,
        nucmer_min_length=options.min_length,
        nucmer_min_length_for_merges=options.min_length_merge,
        nucmer_breaklen=options.breaklen,
        min_spades_circular_percent=options.min_spades_circ_pc,
        spades_careful=not options.assemble_not_careful,
        spades_only_assembler=not options.assemble_not_only_assembler,
        spades_kmers=options.spades_k,
        spades_use_first_success=options.spades_use_first,
        assembler=options.assembler,
        length_cutoff=options.b2r_length_cutoff,
        split_all_reads=options.b2r_split_all_reads,
        data_type=options.data_type,
        ref_end_tolerance=options.ref_end,
        qry_end_tolerance=options.reassemble_end,
        threads=options.threads,
        verbose=options.verbose,
        reads=options.reads,
    )
    m.run()




import unittest
import os
from circlator import common

modules_dir = os.path.dirname(os.path.abspath(common.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestCommon(unittest.TestCase):
    def test_check_files_exist(self):
        '''test check_files_exist'''
        file_exists = os.path.join(data_dir, 'common_test_file_exists')
        common.check_files_exist([file_exists])
        with self.assertRaises(common.Error):
            common.check_files_exist([file_exists, 'thisisnotafileandshouldcauseanerror'])



import unittest
import filecmp
import os
import pyfastaq
from circlator import bamfilter

modules_dir = os.path.dirname(os.path.abspath(bamfilter.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestBamfilter(unittest.TestCase):
    def test_get_ref_lengths(self):
        '''test _get_ref_lengths'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_get_ref_lengths.bam'), 'out')
        expected = {
            'ref1': 41,
            'ref2': 42,
            'ref3': 43,
        }
        self.assertEqual(expected, b._get_ref_lengths())


    def test_get_contigs_to_use(self):
        '''test _get_contigs_to_use'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_get_contigs_to_use.bam'), 'out')
        test_file = os.path.join(data_dir, 'bamfilter_test_get_contigs_to_use.infile')
        self.assertEqual(b._get_contigs_to_use(test_file), {'contig42', 'contig4444244'})
        self.assertEqual(b._get_contigs_to_use(None), set())
        self.assertEqual(b._get_contigs_to_use({'42', '43'}), {'42', '43'})


    def test_check_contigs_to_use(self):
        '''test _check_contigs_to_use'''
        input_bam = os.path.join(data_dir, 'bamfilter_test_check_contigs_to_use.bam')
        b = bamfilter.BamFilter(input_bam, 'out')
        ref_lengths = b._get_ref_lengths()
        self.assertTrue(b._check_contigs_to_use(ref_lengths))

        b = bamfilter.BamFilter(input_bam, 'out', contigs_to_use={'1'})
        self.assertTrue(b._check_contigs_to_use(ref_lengths))

        b = bamfilter.BamFilter(input_bam, 'out', contigs_to_use={'1', '2'})
        self.assertTrue(b._check_contigs_to_use(ref_lengths))

        with self.assertRaises(bamfilter.Error):
            b = bamfilter.BamFilter(input_bam, 'out', contigs_to_use={'42'})
            self.assertTrue(b._check_contigs_to_use(ref_lengths))


    def test_all_reads_from_contig(self):
        '''test _all_reads_from_contig'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_all_reads_from_contig.bam'), 'out')
        tmp = 'tmp.test_all_reads_from_contig.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        expected = os.path.join(data_dir, 'bamfilter_test_all_reads_from_contig.reads.fa')
        b._all_reads_from_contig('1', f)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_get_all_unmapped_reads(self):
        '''test _get_all_unmapped_reads'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_get_all_unmapped_reads.bam'), 'out')
        expected = os.path.join(data_dir, 'bamfilter_test_get_all_unmapped_reads.reads.fa')
        tmp = 'tmp.test_get_all_unmapped_reads.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        b._get_all_unmapped_reads(f)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_break_reads(self):
        '''test _break_reads'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_break_reads.bam'), 'out')
        expected = os.path.join(data_dir, 'bamfilter_test_break_reads.broken_reads.fa')
        tmp = 'tmp.test_break_reads.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        b._break_reads('contig1', 390, f, min_read_length=5)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_exclude_region(self):
        '''test _exclude_region'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_exclude_region.bam'), 'out')
        expected = os.path.join(data_dir, 'bamfilter_test_exclude_region.reads.fa')
        tmp = 'tmp.test_exclude_reads.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        b._exclude_region('1', 500, 700, f)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_get_region_start(self):
        '''test _get_region start'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_get_region_start.bam'), 'out')
        expected = os.path.join(data_dir, 'bamfilter_test_get_region_start.reads.fa')
        tmp = 'tmp.test_get_region.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        b._get_region('1', 0, 64, f, min_length=20)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_get_region_end(self):
        '''test _get_region end'''
        b = bamfilter.BamFilter(os.path.join(data_dir, 'bamfilter_test_get_region_end.bam'), 'out')
        expected = os.path.join(data_dir, 'bamfilter_test_get_region_end.reads.fa')
        tmp = 'tmp.test_get_region.out.fa'
        f = pyfastaq.utils.open_file_write(tmp)
        b._get_region('2', 379, 499, f, min_length=20)
        pyfastaq.utils.close(f)
        self.assertTrue(filecmp.cmp(expected, tmp, shallow=False))
        os.unlink(tmp)


    def test_run_keep_unmapped_no_quals(self):
        '''test run keep unmapped bam has no quality scores'''
        outprefix = 'tmp.bamfilter_run'
        b = bamfilter.BamFilter(
            os.path.join(data_dir, 'bamfilter_test_run_no_qual.bam'),
            outprefix,
            length_cutoff=600,
            min_read_length=100,
            contigs_to_use={'contig1', 'contig3', 'contig4'}
        )
        b.run()
        expected = os.path.join(data_dir, 'bamfilter_test_run_keep_unmapped.out.reads.fa')
        self.assertTrue(filecmp.cmp(expected, outprefix + '.fasta', shallow=False))
        os.unlink(outprefix + '.fasta')
        os.unlink(outprefix + '.log')

        b = bamfilter.BamFilter(
            os.path.join(data_dir, 'bamfilter_test_run_no_qual.bam'),
            outprefix,
            fastq_out=True,
            length_cutoff=600,
            min_read_length=100,
            contigs_to_use={'contig1', 'contig3', 'contig4'}
        )
        b.run()
        expected = os.path.join(data_dir, 'bamfilter_test_run_keep_unmapped.out.reads.fa')
        self.assertTrue(filecmp.cmp(expected, outprefix + '.fastq', shallow=False))
        os.unlink(outprefix + '.fastq')
        os.unlink(outprefix + '.log')


    def test_run_keep_unmapped_with_quals(self):
        '''test run keep unmapped bam has quality scores'''
        outprefix = 'tmp.bamfilter_run'
        b = bamfilter.BamFilter(
            os.path.join(data_dir, 'bamfilter_test_run_with_qual.bam'),
            outprefix,
            fastq_out=False,
            length_cutoff=600,
            min_read_length=100,
            contigs_to_use={'contig1', 'contig3', 'contig4'}
        )
        b.run()
        expected = os.path.join(data_dir, 'bamfilter_test_run_keep_unmapped.out.reads.fa')
        self.assertTrue(filecmp.cmp(expected, outprefix + '.fasta', shallow=False))
        os.unlink(outprefix + '.fasta')
        os.unlink(outprefix + '.log')

        b = bamfilter.BamFilter(
            os.path.join(data_dir, 'bamfilter_test_run_with_qual.bam'),
            outprefix,
            fastq_out=True,
            length_cutoff=600,
            min_read_length=100,
            contigs_to_use={'contig1', 'contig3', 'contig4'}
        )
        b.run()
        expected = os.path.join(data_dir, 'bamfilter_test_run_keep_unmapped.out.reads.fq')
        self.assertTrue(filecmp.cmp(expected, outprefix + '.fastq', shallow=False))
        os.unlink(outprefix + '.fastq')
        os.unlink(outprefix + '.log')


    def test_run_discard_unmapped_no_quals(self):
        '''test run keep unmapped bam has no quality scores'''
        outprefix = 'tmp.bamfilter_run'
        b = bamfilter.BamFilter(
            os.path.join(data_dir, 'bamfilter_test_run_no_qual.bam'),
            outprefix,
            length_cutoff=600,
            min_read_length=100,
            contigs_to_use={'contig1', 'contig3', 'contig4'},
            discard_unmapped=True
        )
        b.run()
        expected = os.path.join(data_dir, 'bamfilter_test_run_discard_unmapped.out.reads.fa')
        self.assertTrue(filecmp.cmp(expected, outprefix + '.fasta', shallow=False))
        os.unlink(outprefix + '.fasta')
        os.unlink(outprefix + '.log')




import unittest
import shutil
import filecmp
import copy
import os
import pymummer
import pyfastaq
from circlator import merge

modules_dir = os.path.dirname(os.path.abspath(merge.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestMerge(unittest.TestCase):
    def setUp(self):
        self.merger = merge.Merger(
            os.path.join(data_dir, 'merge_test_original.fa'),
            os.path.join(data_dir, 'merge_test_reassembly.fa'),
            'tmp.merge_test'
        )


    def test_load_nucmer_hits(self):
        '''test _load_nucmer_hits'''
        got = self.merger._load_nucmer_hits(os.path.join(data_dir, 'merge_test_load_nucmer_hits.coords'))
        lines = [
            '\t'.join(['61', '500', '61', '500', '440', '440', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['10', '50', '11', '52', '51', '52', '99.42', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '500', '1', '499', '500', '499', '99.40', '500', '499', '1', '1', 'ref2', 'qry2', '[IDENTITY]'])
        ]
        expected = {
            'ref1': [
                pymummer.alignment.Alignment(lines[0]),
                pymummer.alignment.Alignment(lines[1]),
            ],
            'ref2': [
                pymummer.alignment.Alignment(lines[2]),
            ]
        }
        self.assertEqual(got, expected)


    def test_hits_hashed_by_query(self):
        '''test _hits_hashed_by_query'''
        hits = [
            '\t'.join(['61', '500', '61', '500', '440', '440', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['10', '50', '11', '52', '51', '52', '99.42', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '500', '1', '499', '500', '499', '99.40', '500', '499', '1', '1', 'ref2', 'qry2', '[IDENTITY]'])
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = {
            'qry1': [hits[0], hits[1]],
            'qry2': [hits[2]]
        }
        got = self.merger._hits_hashed_by_query(hits)
        self.assertEqual(got, expected)


    def test_get_longest_hit_by_ref_length(self):
        '''test _get_longest_hit_by_ref_length'''
        hits = [
            '\t'.join(['42', '542', '43', '543', '500', '502', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['42', '542', '43', '543', '501', '500', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['42', '542', '43', '543', '499', '500', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[1]
        got = self.merger._get_longest_hit_by_ref_length(hits)
        self.assertEqual(expected, got)


    def test_is_at_ref_start(self):
        '''test _is_at_ref_start'''
        hits_at_start = [
            '\t'.join(['42', '542', '43', '543', '501', '500', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['15000', '15542', '1', '543', '542', '500', '100.00', '500', '500', '1', '1', 'ref1', 'qry1']),
        ]
        hits_at_start = [pymummer.alignment.Alignment(x) for x in hits_at_start]

        hits_not_at_start = [
            '\t'.join(['19001', '20000', '1', '1000', '1000', '1000', '100.00', '20000', '500', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['15001', '15542', '1', '543', '542', '542', '100.00', '20000', '500', '1', '1', 'ref1', 'qry1']),
        ]
        hits_not_at_start = [pymummer.alignment.Alignment(x) for x in hits_not_at_start]

        for hit in hits_at_start:
            self.assertTrue(self.merger._is_at_ref_start(hit))

        for hit in hits_not_at_start:
            self.assertFalse(self.merger._is_at_ref_start(hit))


    def test_is_at_ref_end(self):
        '''test _is_at_ref_end'''
        hits_at_end = [
            '\t'.join(['19000', '20000', '1', '1000', '1000', '1000', '100.00', '20000', '1000', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['3999', '5001', '1', '1000', '1000', '1000', '100.00', '20000', '1000', '1', '1', 'ref1', 'qry1']),
        ]
        hits_at_end = [pymummer.alignment.Alignment(x) for x in hits_at_end]

        hits_not_at_end = [
            '\t'.join(['1000', '2000', '1', '1000', '1000', '1000', '100.00', '20000', '1000', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['3999', '5000', '1', '1000', '1000', '1000', '100.00', '20000', '1000', '1', '1', 'ref1', 'qry1']),
        ]
        hits_not_at_end = [pymummer.alignment.Alignment(x) for x in hits_not_at_end]

        for hit in hits_at_end:
            self.assertTrue(self.merger._is_at_ref_end(hit))

        for hit in hits_not_at_end:
            self.assertFalse(self.merger._is_at_ref_end(hit))


    def test_is_at_qry_start(self):
        '''test _is_at_qry_start'''
        hits_at_start = [
            '\t'.join(['42', '1042', '1', '1000', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '1000', '1000', '2000', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits_at_start = [pymummer.alignment.Alignment(x) for x in hits_at_start]

        hits_not_at_start = [
            '\t'.join(['1', '1000', '1001', '2000', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits_not_at_start = [pymummer.alignment.Alignment(x) for x in hits_not_at_start]

        for hit in hits_at_start:
            self.assertTrue(self.merger._is_at_qry_start(hit))

        for hit in hits_not_at_start:
            self.assertFalse(self.merger._is_at_qry_start(hit))


    def test_is_at_qry_end(self):
        '''test _is_at_qry_end'''
        hits_at_end = [
            '\t'.join(['42', '1042', '3242', '4242', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['42', '1042', '2224', '3243', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits_at_end = [pymummer.alignment.Alignment(x) for x in hits_at_end]

        hits_not_at_end = [
            '\t'.join(['1', '1000', '2224', '3242', '1000', '1000', '100.00', '2000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits_not_at_end = [pymummer.alignment.Alignment(x) for x in hits_not_at_end]

        for hit in hits_at_end:
            self.assertTrue(self.merger._is_at_qry_end(hit))

        for hit in hits_not_at_end:
            self.assertFalse(self.merger._is_at_qry_end(hit))


    def test_get_hit_nearest_ref_start(self):
        '''test _get_hit_nearest_ref_start'''
        hits = [
            '\t'.join(['42', '1042', '1', '1000', '1001', '1001', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '1000', '1000', '2000', '1000', '1000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['16000', '18000', '2000', '4000', '2000', '2000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[1]
        self.assertEqual(expected, self.merger._get_hit_nearest_ref_start(hits))


    def test_get_hit_nearest_ref_end(self):
        '''test _get_hit_nearest_ref_end'''
        hits = [
            '\t'.join(['42', '1042', '1', '1000', '1001', '1001', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['16000', '18000', '2000', '4000', '2000', '2000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '1000', '1000', '2000', '1000', '1000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[1]
        self.assertEqual(expected, self.merger._get_hit_nearest_ref_end(hits))


    def test_get_longest_hit_at_ref_start(self):
        '''test _get_longest_hit_at_ref_start'''
        hits = [
            '\t'.join(['42', '1042', '1', '1000', '1001', '1001', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '1000', '1000', '2000', '1000', '1000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['16000', '18000', '2000', '4000', '2000', '2000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[0]
        self.assertEqual(expected, self.merger._get_longest_hit_at_ref_start(hits))


    def test_get_longest_hit_at_ref_end(self):
        '''test _get_longest_hit_at_ref_end'''
        hits = [
            '\t'.join(['15000', '18001', '1', '3000', '3001', '3001', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['15001', '18001', '1', '3000', '3000', '3000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '4000', '1000', '4000', '4000', '4000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[0]
        self.assertEqual(expected, self.merger._get_longest_hit_at_ref_end(hits))


    def test_get_longest_hit_at_qry_start(self):
        '''test _get_longest_hit_at_qry_start'''
        hits = [
            '\t'.join(['42', '1042', '1', '1000', '1001', '1001', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['42', '1042', '1', '999', '1000', '1000', '100.00', '20000', '4242', '1', '1', 'ref2', 'qry1']),
            '\t'.join(['16000', '18000', '2000', '4000', '2000', '2000', '100.00', '20000', '4242', '1', '1', 'ref1', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[0]
        self.assertEqual(expected, self.merger._get_longest_hit_at_qry_start(hits))


    def test_get_longest_hit_at_qry_end(self):
        '''test _get_longest_hit_at_qry_end'''
        hits = [
            '\t'.join(['1', '1000', '40000', '42000', '2000', '2000', '100.00', '20000', '42000', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['1', '1001', '39999', '42000', '2001', '2001', '100.00', '20000', '42000', '1', '1', 'ref2', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        expected = hits[1]
        self.assertEqual(expected, self.merger._get_longest_hit_at_qry_end(hits))


    def test_hits_have_same_query(self):
        '''test _hits_have_same_query'''
        hits = [
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref42', 'qry42']),
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref43', 'qry42']),
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref44', 'qry43']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        self.assertTrue(self.merger._hits_have_same_query(hits[0], hits[1]))
        self.assertFalse(self.merger._hits_have_same_query(hits[0], hits[2]))


    def test_hits_have_same_reference(self):
        '''test _hits_have_same_reference'''
        hits = [
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref42', 'qry42']),
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref42', 'qry43']),
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref43', 'qry44']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        self.assertTrue(self.merger._hits_have_same_reference(hits[0], hits[1]))
        self.assertFalse(self.merger._hits_have_same_reference(hits[0], hits[2]))


    def test_min_qry_hit_length(self):
        '''test _min_qry_hit_length'''
        hits = [
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref42', 'qry42']),
            '\t'.join(['42', '44', '42', '44', '3', '3', '100.00', '424242', '4242', '1', '1', 'ref43', 'qry42']),
            '\t'.join(['42', '45', '42', '45', '4', '4', '100.00', '424242', '4242', '1', '1', 'ref44', 'qry43']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        self.assertEqual(2, self.merger._min_qry_hit_length(hits))


    def test_has_qry_hit_longer_than(self):
        '''test _has_qry_hit_longer_than'''
        hits = [
            '\t'.join(['42', '43', '42', '43', '2', '2', '100.00', '424242', '4242', '1', '1', 'ref42', 'qry42']),
            '\t'.join(['42', '44', '42', '44', '3', '3', '100.00', '424242', '4242', '1', '1', 'ref43', 'qry42']),
            '\t'.join(['42', '45', '42', '45', '4', '4', '100.00', '424242', '4242', '1', '1', 'ref44', 'qry43']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        self.assertTrue(self.merger._has_qry_hit_longer_than(hits, 3))
        self.assertTrue(self.merger._has_qry_hit_longer_than(hits, 3, hits_to_exclude={hits[1]}))
        self.assertFalse(self.merger._has_qry_hit_longer_than(hits, 3, hits_to_exclude={hits[2]}))
        self.assertFalse(self.merger._has_qry_hit_longer_than(hits, 4))


    def test_can_circularise(self):
        '''test _can_circularise'''
        start_hits = [
            '\t'.join(['1', '1000', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['1', '1000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['1000', '1', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['1000', '1', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
        ]
        end_hits = [
            '\t'.join(['419000', '420000', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['419000', '420000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['420000', '419000', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['420000', '419000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
        ]
        start_hits = [pymummer.alignment.Alignment(x) for x in start_hits]
        end_hits = [pymummer.alignment.Alignment(x) for x in end_hits]

        good_pairs = {(1, 0), (2, 3)}

        for i in range(len(start_hits)):
            for j in range(len(end_hits)):
                expected = (i, j) in good_pairs
                got = self.merger._can_circularise(start_hits[i], end_hits[j])
                self.assertEqual(expected, got)


    def test_get_possible_circular_ref_contigs(self):
        '''test _get_possible_circular_ref_contigs'''
        hits = [
            '\t'.join(['1', '1000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['419000', '420000', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref42', 'qry1']),
            '\t'.join(['420000', '419000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref2', 'qry2']),
            '\t'.join(['1', '1000', '3000', '4000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref43', 'qry1']),
            '\t'.join(['419000', '420000', '1', '1000', '1000', '1000', '100.00', '420000', '4000', '1', '1', 'ref43', 'qry1']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        all_hits = {
            'ref42': [hits[0], hits[1]],
            'ref2': [hits[2]],
            'ref43': [hits[3], hits[4]],
        }

        expected = {
            'ref42': (hits[0], hits[1]),
            'ref43': (hits[3], hits[4]),
        }

        got = self.merger._get_possible_circular_ref_contigs(all_hits)
        self.assertEqual(expected, got)


    def test_get_possible_circular_ref_contigs_same_best_hits(self):
        '''test _get_possible_circular_ref_contigs same best hits'''
        hits = [
            '\t'.join(['1', '37978', '37978', '1', '37978', '37978', '99.98', '2177527', '68234', '1', '1', 'ref', 'qry']),
            '\t'.join(['2137113', '2177527', '68234', '27820', '40415', '40415', '99.95', '2177527', '68234', '1', '1', 'ref', 'qry']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        all_hits = {'ref': hits}
        expected = {'ref': (hits[0], hits[1])}
        got = self.merger._get_possible_circular_ref_contigs(all_hits)
        self.assertEqual(expected, got)


    def test_remove_keys_from_dict_with_nonunique_values(self):
        '''test _remove_keys_from_dict_with_nonunique_values'''
        d = {1:1, 2:1, 3:42, 42:43}
        expected = {3:42, 42:43}
        got = self.merger._remove_keys_from_dict_with_nonunique_values(d)
        self.assertEqual(expected, got)


    def test_make_circularised_contig_1(self):
        '''test _make_circularised_contig 1'''
        hits = [
            '\t'.join(['5', '360', '334', '689', '356', '356', '100.00', '817', '689', '1', '1', 'ref', 'reassembly']),
            '\t'.join(['481', '813', '1', '333', '333', '333', '100.00', '817', '689', '1', '1', 'ref', 'reassembly'])
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]
        ref_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.1.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.1.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.1.expected.fa')
        self.merger.original_contigs = {}
        self.merger.reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, self.merger.original_contigs)
        pyfastaq.tasks.file_to_dict(reassembly_fasta, self.merger.reassembly_contigs)
        got = self.merger._make_circularised_contig(start_hit, end_hit)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, got.seq)


    def test_make_circularised_contig_2(self):
        '''test _make_circularised_contig 2'''
        # same as test_make_circularised_contig_1 but reassembly is reverse complemented
        hits = [
            '\t'.join(['5', '360', '356', '1', '356', '356', '100.00', '817', '689', '1', '-1', 'ref', 'reassembly']),
            '\t'.join(['481', '813', '689', '357', '333', '333', '100.00', '817', '689', '1', '-1', 'ref', 'reassembly']),
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]
        ref_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.2.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.2.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_hits_to_new_seq.2.expected.fa')
        self.merger.original_contigs = {}
        self.merger.reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, self.merger.original_contigs)
        pyfastaq.tasks.file_to_dict(reassembly_fasta, self.merger.reassembly_contigs)
        got = self.merger._make_circularised_contig(start_hit, end_hit)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, got.seq)


    def test_orientation_ok_to_bridge_contigs(self):
        '''test _orientation_ok_to_bridge_contigs'''
        hits = [
            '\t'.join(['15000', '20000', '1', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry']),
            '\t'.join(['1', '5000', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry']),
            '\t'.join(['15000', '20000', '5000', '1', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry']),
            '\t'.join(['5000', '1', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry']),
            '\t'.join(['1', '5000', '5000', '1', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry']),
            '\t'.join(['15000', '20000', '10000', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        self.assertTrue(self.merger._orientation_ok_to_bridge_contigs(hits[0], hits[1]))
        self.assertFalse(self.merger._orientation_ok_to_bridge_contigs(hits[0], hits[0]))
        self.assertFalse(self.merger._orientation_ok_to_bridge_contigs(hits[0], hits[2]))
        self.assertFalse(self.merger._orientation_ok_to_bridge_contigs(hits[0], hits[3]))
        self.assertTrue(self.merger._orientation_ok_to_bridge_contigs(hits[4], hits[5]))


    def test_get_possible_query_bridging_contigs(self):
        '''test _get_possible_query_bridging_contigs'''
        hits = [
            '\t'.join(['15000', '20000', '1', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry1']),
            '\t'.join(['1', '5000', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry1']),
            '\t'.join(['1', '5000', '5000', '1', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry2']),
            '\t'.join(['15000', '20000', '10000', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry2']),
            '\t'.join(['15500', '20000', '10000', '5500', '4500', '4500', '100.00', '20000', '10000', '1', '-1', 'ref3', 'qry2']),
            '\t'.join(['15500', '20000', '10000', '5500', '4500', '4500', '100.00', '20000', '10000', '1', '-1', 'ref4', 'qry3']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        input_hits = {
            'qry1': [hits[0], hits[1]],
            'qry2': [hits[2], hits[3], hits[4]],
            'qry3': [hits[5]]
        }
        got = self.merger._get_possible_query_bridging_contigs(input_hits)
        expected = {
            'qry1': (hits[0], hits[1]),
            'qry2': (hits[2], hits[3])
        }
        self.assertEqual(got, expected)


    def test_filter_bridging_contigs(self):
        '''test _filter_bridging_contigs'''
        hits = [
            '\t'.join(['15000', '20000', '1', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref1', 'qry1']),
            '\t'.join(['1', '5000', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref2', 'qry1']),
            '\t'.join(['15000', '20000', '1', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref3', 'qry2']),
            '\t'.join(['1', '5000', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref4', 'qry2']),
            '\t'.join(['15000', '20000', '1', '5000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref3', 'qry3']),
            '\t'.join(['1', '5000', '5000', '10000', '5000', '5000', '100.00', '20000', '10000', '1', '-1', 'ref5', 'qry3']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        input_hits = {
            'qry1': (hits[0], hits[1]),
            'qry2': (hits[2], hits[3]),
            'qry3': (hits[4], hits[5]),
        }
        got = self.merger._filter_bridging_contigs(input_hits)
        expected = {'qry1': (hits[0], hits[1])}
        self.assertEqual(got, expected)


    def test_merge_bridged_contig_pair1(self):
        '''test _merge_bridged_contig_pair1'''
        # simple case: ref contigs 1 and 2 do not overlap each other and are joined by a reassembly contig
        hits = [
            '\t'.join(['721', '999', '5', '283', '279', '279', '100.00', '1000', '753', '1', '1', 'ref1', 'reassembly']),
            '\t'.join(['1', '420', '324', '743', '420', '420', '100.00', '1000', '753', '1', '1', 'ref2', 'reassembly']),
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]

        ref_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test1.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test1.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test1.expected.fa')
        ref_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, ref_contigs)
        reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(reassembly_fasta, reassembly_contigs)
        self.merger._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, reassembly_contigs)
        self.assertEqual(len(ref_contigs), 1)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, list(ref_contigs.values())[0].seq)


    def test_merge_bridged_contig_pair2(self):
        '''test _merge_bridged_contig_pair2'''
        # simple case: ref contigs 1 and 2 do not overlap each other and are joined by a reassembly contig
        hits = [
            '\t'.join(['1', '420', '430', '11', '420', '420', '100.00', '1000', '753', '1', '-1', 'ref2', 'reassembly']),
            '\t'.join(['721', '999', '749', '471', '279', '279', '100.00', '1000', '753', '1', '-1', 'ref1', 'reassembly']),
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]

        ref_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test2.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test2.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test2.expected.fa')
        ref_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, ref_contigs)
        reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(reassembly_fasta, reassembly_contigs)
        self.merger._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, reassembly_contigs)
        self.assertEqual(len(ref_contigs), 1)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, list(ref_contigs.values())[0].seq)


    def test_merge_bridged_contig_pair3(self):
        '''test _merge_bridged_contig_pair3'''
        # simple case: ref contigs 1 and 2 do not overlap each other and are joined by a reassembly contig
        hits = [
            '\t'.join(['541', '960', '1', '420', '420', '420', '100.00', '960', '840', '1', '1', 'ref1', 'reassembly']),
            '\t'.join(['1', '480', '361', '840', '480', '480', '100.00', '1060', '840', '1', '1', 'ref2', 'reassembly']),
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]

        ref_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test3.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test3.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test3.expected.fa')
        ref_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, ref_contigs)
        reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(reassembly_fasta, reassembly_contigs)
        self.merger._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, reassembly_contigs)
        self.assertEqual(len(ref_contigs), 1)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, list(ref_contigs.values())[0].seq)


    def test_merge_bridged_contig_pair4(self):
        '''test _merge_bridged_contig_pair4'''
        # simple case: ref contigs 1 and 2 do not overlap each other and are joined by a reassembly contig
        hits = [
            '\t'.join(['1', '480', '480', '1', '480', '480', '100.00', '1060', '840', '1', '-1', 'ref2', 'reassembly']),
            '\t'.join(['541', '960', '840', '421', '420', '420', '100.00', '960', '840', '1', '-1', 'ref1', 'reassembly']),
        ]
        start_hit, end_hit = [pymummer.alignment.Alignment(x) for x in hits]

        ref_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test4.ref.fa')
        reassembly_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test4.reassembly.fa')
        expected_fasta = os.path.join(data_dir, 'merge_test_merge_bridged_contig_pair.test4.expected.fa')
        ref_contigs = {}
        pyfastaq.tasks.file_to_dict(ref_fasta, ref_contigs)
        reassembly_contigs = {}
        pyfastaq.tasks.file_to_dict(reassembly_fasta, reassembly_contigs)
        self.merger._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, reassembly_contigs)
        self.assertEqual(len(ref_contigs), 1)
        expected_ref_contigs = {}
        pyfastaq.tasks.file_to_dict(expected_fasta, expected_ref_contigs)
        self.assertEqual(list(expected_ref_contigs.values())[0].seq, list(ref_contigs.values())[0].seq)


    def test_index_fasta(self):
        '''test _index_fasta'''
        fasta_file = os.path.join(data_dir, 'merge_test_index_fasta.fa')
        expected_fai = os.path.join(data_dir, 'merge_test_index_fasta.fa.fai')
        test_fa = 'tmp.test_index_fasta.fa'
        test_fai = test_fa + '.fai'
        shutil.copyfile(fasta_file, test_fa)
        self.merger._index_fasta(test_fa)
        self.assertTrue(os.path.exists(test_fai))
        self.assertTrue(filecmp.cmp(expected_fai, test_fai, shallow=False))
        os.unlink(test_fa)
        os.unlink(test_fai)


    def test_write_act_files(self):
        '''test _write_act_files'''
        original_ref_fasta = os.path.join(data_dir, 'merge_test_write_act_files.ref.fa')
        original_qry_fasta = os.path.join(data_dir, 'merge_test_write_act_files.qry.fa')
        original_coords_file = os.path.join(data_dir, 'merge_test_write_act_files.coords')
        ref_fasta = 'tmp.test_write_act_files.ref.fa'
        qry_fasta = 'tmp.test_write_act_files.qry.fa'
        coords_file = 'tmp.test_write_act_files.coords'
        shutil.copyfile(original_ref_fasta, ref_fasta)
        shutil.copyfile(original_qry_fasta, qry_fasta)
        shutil.copyfile(original_coords_file, coords_file)

        expected_bash_script = os.path.join(data_dir, 'merge_test_write_act_files.expected.sh')
        expected_crunch_file = os.path.join(data_dir, 'merge_test_write_act_files.expected.crunch')
        outprefix = 'tmp.test_write_act_files.out'
        got_bash_script = outprefix + '.start_act.sh'
        got_crunch_file = outprefix + '.crunch'

        self.merger._write_act_files(ref_fasta, qry_fasta, coords_file, outprefix)
        self.assertTrue(filecmp.cmp(expected_bash_script, got_bash_script, shallow=False))
        self.assertTrue(filecmp.cmp(expected_crunch_file, got_crunch_file, shallow=False))
        for f in [ref_fasta, ref_fasta + '.fai', qry_fasta, qry_fasta + '.fai', coords_file, got_bash_script, got_crunch_file]:
            os.unlink(f)


    def test_contigs_dict_to_file(self):
        '''test _contigs_dict_to_file'''
        d = {
            '3': pyfastaq.sequences.Fasta('3', 'A'),
            '2': pyfastaq.sequences.Fasta('2', 'AAC'),
            '1': pyfastaq.sequences.Fasta('1', 'ACGTA'),
        }
        tmpfile = 'tmp.test_contigs_dict_to_file.fa'
        self.merger._contigs_dict_to_file(d, tmpfile)
        self.assertTrue(filecmp.cmp(tmpfile, os.path.join(data_dir, 'merge_test_contigs_dict_to_file.fa'), shallow=False))
        os.unlink(tmpfile)


    def test_make_new_contig_from_nucmer_and_spades_not_hit(self):
        '''test _make_new_contig_from_nucmer_and_spades no hit'''
        circular = set()
        hits = [
            '\t'.join(['1', '42', '2', '43', '42', '42', '100.0', '1000', '2000', '1', '-1', 'ref', 'reassembly'])
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        got = self.merger._make_new_contig_from_nucmer_and_spades('contig_name', hits, circular)
        self.assertEqual(got, (None, None))


    def test_make_new_contig_from_nucmer_and_spades_with_hit(self):
        '''test _make_new_contig_from_nucmer_and_spades with hit'''
        circular = {'spades_node'}
        spades_node = pyfastaq.sequences.Fasta('spades_node', 'ACGTACGTACG')
        expected = pyfastaq.sequences.Fasta('contig_name', spades_node.seq), 'spades_node'
        self.merger.reassembly_contigs = {'spades_node': spades_node}
        hits = [
            '\t'.join(['1', '10', '1', '11', '11', '11', '100.0', '30', '11', '1', '-1', 'original', 'spades_node']),
            '\t'.join(['21', '30', '1', '11', '11', '11', '100.0', '30', '11', '1', '-1', 'original', 'spades_node']),
            '\t'.join(['11', '20', '1', '11', '11', '11', '100.0', '30', '11', '1', '-1', 'original', 'spades_node']),
        ]
        hits = [pymummer.alignment.Alignment(x) for x in hits]
        got = self.merger._make_new_contig_from_nucmer_and_spades('contig_name', hits, circular)
        self.assertEqual(got, expected)


    def test_get_spades_circular_nodes(self):
        fastg = os.path.join(data_dir, 'merge_test_get_spades_circular_nodes.fastg')
        got = self.merger._get_spades_circular_nodes(fastg)
        expected = set(['NODE_1_length_5_cov_42.42_ID_1'])
        self.assertEqual(expected, got)




import os
import tempfile
import sys
import shutil
import pyfastaq
from circlator import common, external_progs

class Error (Exception): pass

class Assembler:
    def __init__(self,
      reads,
      outdir,
      threads=1,
      spades_kmers=None,
      careful=True,
      only_assembler=True,
      verbose=False,
      spades_use_first_success=False,
      assembler='spades',
      genomeSize=100000, # only matters for Canu if correcting reads (which we're not)
      data_type='pacbio-corrected',
    ):
        self.outdir = os.path.abspath(outdir)
        self.reads = os.path.abspath(reads)
        if not os.path.exists(self.reads):
            raise Error('Reads file not found:' + self.reads)

        self.verbose = verbose
        self.samtools = external_progs.make_and_check_prog('samtools', verbose=self.verbose)
        self.threads = threads
        self.assembler = assembler

        if self.assembler == 'spades':
            self.spades = external_progs.make_and_check_prog('spades', verbose=self.verbose, required=True)
            self.spades_kmers = self._build_spades_kmers(spades_kmers)
            self.spades_use_first_success = spades_use_first_success
            self.careful = careful
            self.only_assembler = only_assembler
        elif self.assembler == 'canu':
            self.canu = external_progs.make_and_check_prog('canu', verbose=self.verbose, required=True)
            self.genomeSize=genomeSize
            self.data_type = data_type
        else:
            raise Error('Unknown assembler: "' + self.assembler + '". cannot continue')



    def _build_spades_kmers(self, kmers):
        if kmers is None:
            return [127,117,107,97,87,77]
        elif type(kmers) == str:
            try:
                kmer_list = [int(k) for k in kmers.split(',')]
            except:
                raise Error('Error getting list of kmers from:' + str(kmers))
            return kmer_list
        elif type(kmers) == list:
            return kmers
        else:
            raise Error('Error getting list of kmers from:' + str(kmers))


    def _make_spades_command(self, kmer, outdir):
        cmd = [
            self.spades.exe(),
            '-s', self.reads,
            '-o', outdir,
            '-t', str(self.threads),
            '-k', str(kmer),
        ]

        if self.careful:
            cmd.append('--careful')

        if self.only_assembler:
            cmd.append('--only-assembler')

        return ' '.join(cmd)


    def _make_canu_command(self, outdir, out_name):
        cmd = [
            self.canu.exe(),
            '-useGrid=false',
            'gnuplotTested=true',
            '-assemble',
            'genomeSize='+str(float(self.genomeSize)/1000000)+'m',
            '-d', outdir,
            '-p', out_name,
            '-'+self.data_type,
            self.reads,
        ]
        return ' '.join(cmd)


    def run_spades_once(self, kmer, outdir):
        cmd = self._make_spades_command(kmer, outdir)
        return common.syscall(cmd, verbose=self.verbose, allow_fail=True)


    def run_spades(self, stop_at_first_success=False):
        '''Runs spades on all kmers. Each a separate run because SPAdes dies if any kmer does
           not work. Chooses the 'best' assembly to be the one with the biggest N50'''
        n50 = {}
        kmer_to_dir = {}

        for k in self.spades_kmers:
            tmpdir = tempfile.mkdtemp(prefix=self.outdir + '.tmp.spades.' + str(k) + '.', dir=os.getcwd())
            kmer_to_dir[k] = tmpdir
            ok, errs = self.run_spades_once(k, tmpdir)
            if ok:
                contigs_fasta = os.path.join(tmpdir, 'contigs.fasta')
                contigs_fai = contigs_fasta + '.fai'
                common.syscall(self.samtools.exe() + ' faidx ' + contigs_fasta, verbose=self.verbose)
                stats = pyfastaq.tasks.stats_from_fai(contigs_fai)
                if stats['N50'] != 0:
                    n50[k] = stats['N50']

                    if stop_at_first_success:
                        break

        if len(n50) > 0:
            if self.verbose:
                print('[assemble]\tkmer\tN50')
                for k in sorted(n50):
                    print('[assemble]', k, n50[k], sep='\t')

            best_k = None

            for k in sorted(n50):
                if best_k is None or n50[k] >= n50[best_k]:
                    best_k = k

            assert best_k is not None

            for k, directory in kmer_to_dir.items():
                if k == best_k:
                    if self.verbose:
                        print('[assemble] using assembly with kmer', k)
                    os.rename(directory, self.outdir)
                else:
                    shutil.rmtree(directory)
        else:
            raise Error('Error running SPAdes. Output directories are:\n  ' + '\n  '.join(kmer_to_dir.values()) + '\nThe reason why should be in the spades.log file in each directory.')


    @classmethod
    def _rename_canu_contigs(cls, infile, outfile):
        with open (infile) as f_in:
            with open(outfile, 'w') as f_out:
                for line in f_in:
                    if line.startswith('>'):
                        print(line.split()[0], file=f_out)
                    else:
                        print(line, end='', file=f_out)


    def run_canu(self):
        '''Runs canu instead of spades'''
        cmd = self._make_canu_command(self.outdir,'canu')
        ok, errs = common.syscall(cmd, verbose=self.verbose, allow_fail=False)
        if not ok:
            raise Error('Error running Canu.')

        original_contigs = os.path.join(self.outdir, 'canu.contigs.fasta')
        renamed_contigs = os.path.join(self.outdir, 'contigs.fasta')
        Assembler._rename_canu_contigs(original_contigs, renamed_contigs)
        original_gfa = os.path.join(self.outdir, 'canu.contigs.gfa')
        renamed_gfa = os.path.join(self.outdir, 'contigs.gfa')
        os.rename(original_gfa, renamed_gfa)


    def run(self):
        if self.assembler == 'spades':
            self.run_spades(stop_at_first_success=self.spades_use_first_success)
        elif self.assembler == 'canu':
            self.run_canu()
        else:
            raise Error('Unknown assembler: "' + self.assembler + '". cannot continue')



import argparse
import sys
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Map reads using bwa mem',
        usage = 'circlator mapreads [options] <reference.fasta> <reads.fasta> <out.bam>')
    parser.add_argument('--bwa_opts', help='BWA options, in quotes [%(default)s]', default='-x pacbio', metavar='STRING')
    parser.add_argument('--threads', type=int, help='Number of threads [%(default)s]', default=1, metavar='INT')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('ref', help='Name of input reference FASTA file', metavar='reference.fasta')
    parser.add_argument('reads', help='Name of corrected reads FASTA file', metavar='reads.fasta')
    parser.add_argument('bam', help='Name of output BAM file', metavar='out.bam')
    options = parser.parse_args()

    circlator.mapping.bwa_mem(
      options.ref,
      options.reads,
      options.bam,
      threads=options.threads,
      bwa_options=options.bwa_opts,
      verbose=options.verbose,
    )



import argparse
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Clean contigs',
        usage = 'circlator clean [options] <in.fasta> <outprefix>')
    parser.add_argument('--min_contig_length', type=int, help='Contigs shorter than this are discarded (unless specified using --keep) [%(default)s]', default=2000, metavar='INT')
    parser.add_argument('--min_contig_percent', type=int, help='If length of nucmer hit is at least this percentage of length of contig, then contig is removed. (unless specified using --keep) [%(default)s]', default=95, metavar='FLOAT')
    parser.add_argument('--diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    parser.add_argument('--min_nucmer_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    parser.add_argument('--min_nucmer_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    parser.add_argument('--breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)
    parser.add_argument('--keep', help='File of contig names to keep in output file', metavar='FILENAME')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('fasta_in', help='Name of input FASTA file', metavar='in.fasta')
    parser.add_argument('outprefix', help='Prefix of output files')
    options = parser.parse_args()

    cleaner = circlator.clean.Cleaner(
        options.fasta_in,
        options.outprefix,
        min_contig_length=options.min_contig_length,
        min_contig_percent_match=options.min_contig_percent,
        nucmer_diagdiff=options.diagdiff,
        nucmer_min_id=options.min_nucmer_id,
        nucmer_min_length=options.min_nucmer_length,
        nucmer_breaklen=options.breaklen,
        keepfile=options.keep,
        verbose=options.verbose
    )

    cleaner.run()



import os
import shutil
import copy
import tempfile
from distutils.version import LooseVersion
import pyfastaq
import pymummer
import circlator

class Error (Exception): pass

class StartFixer:
    def __init__(self,
        input_assembly_fa,
        outprefix,
        min_percent_identity=70,
        promer_mincluster=None,
        genes_fa=None,
        ignore=None,
        verbose=False,
    ):
        if genes_fa is None:
            d = os.path.abspath(os.path.dirname(circlator.__file__))
            self.genes_fa = os.path.join(d, 'data', 'dnaA.fasta')
            assert os.path.exists(self.genes_fa)
        else:
            if not os.path.exists(genes_fa):
                raise Error('Genes file "' + genes_fa + '" not found. Cannot continue')
            self.genes_fa = os.path.abspath(genes_fa)

        if not os.path.exists(input_assembly_fa):
            raise Error('Error! File not found: ' + input_assembly_fa)
        self.input_assembly = {}
        pyfastaq.tasks.file_to_dict(input_assembly_fa, self.input_assembly)

        self.min_percent_identity = min_percent_identity
        self.promer_mincluster = promer_mincluster
        self.outprefix = os.path.abspath(outprefix)
        self.verbose = verbose

        if ignore is None:
            self.ignore = set()
        else:
            with open(ignore) as f:
                self.ignore = {x.rstrip().split()[0] for x in f}


    @classmethod
    def _rename_contigs(cls, contigs_dict):
        rename_dict = {}
        new_contigs_dict = {}
        for old_name, contig in contigs_dict.items():
            new_name = old_name.split()[0]
            if new_name in rename_dict:
                raise Error('Non-unique name "' + new_name + '" in input contigs. Cannot continue')
            rename_dict[new_name] = old_name
            new_contigs_dict[new_name] = pyfastaq.sequences.Fasta(new_name, contig.seq)

        return new_contigs_dict, rename_dict


    @classmethod
    def _write_renamed_contigs(cls, contigs_dict, rename_dict, outfile):
        f = pyfastaq.utils.open_file_write(outfile)

        for old_name, contig in sorted(contigs_dict.items()):
            contig.id = rename_dict[old_name]
            print(contig, file=f)

        pyfastaq.utils.close(f)


    @classmethod
    def _max_length_from_fasta_file(cls, infile):
        freader = pyfastaq.sequences.file_reader(infile)
        return max([len(x) for x in freader])


    @classmethod
    def _write_fasta_plus_circularized_ends(cls, contigs, outfile, end_length, ignore=None):
        if ignore is None:
            ignore = set()

        if ignore.issuperset(set(contigs.keys())):
            return 0

        f = pyfastaq.utils.open_file_write(outfile)
        used_names = set(contigs.keys())
        seqs_written = 0

        for contig_name, contig in sorted(contigs.items()):
            if contig_name in ignore:
                continue

            print(contig, file=f)
            seqs_written += 1

            if len(contig) >= 2 * end_length:
                start_coord = end_length - 1
                end_coord = len(contig) - end_length
                new_name = contig.id + '__ends'
                assert new_name not in used_names
                used_names.add(new_name)
                new_contig = pyfastaq.sequences.Fasta(new_name, contig[end_coord:] + contig[:start_coord + 1])
                print(new_contig, file=f)
                seqs_written += 1

        pyfastaq.utils.close(f)
        return seqs_written


    @classmethod
    def _find_circular_using_promer(cls, outprefix, ref_genes_fa, contigs_dict, min_percent_id, end_length, log_fh, ignore=None, promer_mincluster=None):
        if ignore is None:
            ignore = set()
        promer_out = outprefix + '.promer'
        contigs_with_ends = outprefix + '.contigs_with_ends.fa'
        sequences_written = StartFixer._write_fasta_plus_circularized_ends(contigs_dict, contigs_with_ends, end_length, ignore=ignore)
        if sequences_written == 0:
            print('No sequences to check with promer. Skipping', file=log_fh)
            return {}

        prunner = pymummer.nucmer.Runner(
            contigs_with_ends,
            ref_genes_fa,
            promer_out,
            min_id=min_percent_id,
            promer=True,
            verbose=False,
            maxmatch=True,
            mincluster=promer_mincluster,
        )
        prunner.run()

        circularized = {} # original_contig_name -> promer match
        file_reader = pymummer.coords_file.reader(promer_out)

        for match in file_reader:
            if match.ref_name.endswith('__ends'):
                original_contig_name = match.ref_name[:-len('__ends')]
            else:
                original_contig_name = match.ref_name

            if match.hit_length_qry == match.qry_length and match.percent_identity >= min_percent_id:
                if original_contig_name not in circularized or circularized[original_contig_name].percent_identity < match.percent_identity:
                    circularized[original_contig_name] = copy.copy(match)

        if len(circularized):
            print('Using the following promer matches to circularize contigs:', file=log_fh)
            for contig_name in circularized:
                print(circularized[contig_name], file=log_fh)
        else:
            print('No suitable promer matches found', file=log_fh)

        return circularized


    @classmethod
    def _find_circular_using_prodigal(cls, outprefix, contigs_dict, circular_from_promer, log_fh, ignore=None):
        if ignore is None:
            ignore = set()

        all_to_ignore = copy.copy(ignore)
        all_to_ignore.update(set(circular_from_promer.keys()))
        if all_to_ignore.issuperset(set(contigs_dict.keys())):
            print('No sequences left for which to look for genes using prodigal. Skipping', file=log_fh)
            return {}

        prodigal = circlator.external_progs.make_and_check_prog('prodigal')
        total_contig_length = 0
        prodigal_input_file = outprefix + '.for_prodigal.fa'
        prodigal_output_file = outprefix + '.prodigal.gff'
        with open(prodigal_input_file, 'w') as f:
            for seq_name in sorted(contigs_dict):
                if seq_name not in circular_from_promer and seq_name not in ignore:
                    total_contig_length += len(contigs_dict[seq_name])
                    print(contigs_dict[seq_name], file=f)


        if (total_contig_length < 20000):
            # prodigal needs -p meta option for sequences less than 20000
            # annoyingly newer version of prodigal has different -p option!
            if LooseVersion(prodigal.version) >= LooseVersion('3.0'):
                p_option = "-p anon"
            else:
                p_option = "-p meta"
        else:
            p_option = ''

        cmd = ' '.join([
          'prodigal',
          '-i', prodigal_input_file,
          '-o', prodigal_output_file,
          '-f gff -c -q',
          p_option
        ])

        circlator.common.syscall(cmd)
        circularized = {}
        best_dist = {}

        with open(prodigal_output_file) as f:
            for line in f:
                if line.startswith('#'):
                    continue

                contig, x, y, start, end, *z = line.rstrip().split('\t')
                new_inter = pyfastaq.intervals.Interval(int(start), int(end))
                new_dist = new_inter.distance_to_point(int(len(contigs_dict[contig])/2))

                if contig in circularized:
                    assert contig in best_dist
                    if new_dist < best_dist[contig]:
                        best_dist[contig] = new_dist
                        circularized[contig] = line.rstrip()
                else:
                    best_dist[contig] = new_dist
                    circularized[contig] = line.rstrip()

        if len(circularized):
            print('Using the following prodigal predictions to circularize contigs:', file=log_fh)
            for contig_name in circularized:
                print(circularized[contig_name], file=log_fh)
        else:
            print('No prodigal matches found', file=log_fh)

        return circularized


    @classmethod
    def _rearrange_contigs(cls, contigs_dict, circular_from_promer, circular_from_prodigal, to_ignore, end_length, log_filename):
        log_fh = pyfastaq.utils.open_file_write(log_filename)
        log_prefix = '[fixstart]'
        print(log_prefix, 'id', 'break_point', 'gene_name', 'gene_reversed', 'new_name', 'skipped', sep='\t', file=log_fh)
        for contig_name, contig in sorted(contigs_dict.items()):
            original_length = len(contig)
            if contig.id in circular_from_promer:
                match = circular_from_promer[contig.id]
                if contig.id == match.ref_name:
                    if match.on_same_strand():
                        position = min(match.ref_start, match.ref_end)
                        contig.seq = contig.seq[position:] + contig.seq[:position]
                        print(log_prefix, contig.id, position + 1, match.qry_name, 'no', '-', '-', sep='\t', file=log_fh)
                    else:
                        position = max(match.ref_start, match.ref_end) + 1
                        contig.seq = contig.seq[position:] + contig.seq[:position]
                        contig.revcomp()
                        print(log_prefix, contig.id, position, match.qry_name, 'yes', '-', '-', sep='\t', file=log_fh)
                else:
                    assert contig.id + '__ends' == match.ref_name
                    if match.on_same_strand():
                        start_in_ends_contig = min(match.ref_start, match.ref_end)
                    else:
                        start_in_ends_contig = max(match.ref_start, match.ref_end) + 1

                    if start_in_ends_contig < end_length:
                        position = len(contig) - end_length + start_in_ends_contig
                    else:
                        position = start_in_ends_contig - end_length

                    if match.on_same_strand():
                        contig.seq = contig.seq[position:] + contig.seq[:position]
                        print(log_prefix, contig.id, position + 1, match.qry_name, 'no', '-', '-', sep='\t', file=log_fh)
                    else:
                        contig.seq = contig.seq[position:] + contig.seq[:position]
                        contig.revcomp()
                        print(log_prefix, contig.id, position, match.qry_name, 'yes', '-', '-', sep='\t', file=log_fh)
            elif contig.id in circular_from_prodigal:
                gff_contig_name, x, y, start, end, score, strand, *z = circular_from_prodigal[contig.id].rstrip().split('\t')
                assert contig.id == gff_contig_name

                if strand == '+':
                    position = int(start) - 1
                    contig.seq = contig.seq[position:] + contig.seq[:position]
                    print(log_prefix, contig.id, position + 1, 'prodigal', 'no', '-', '-', sep='\t', file=log_fh)
                else:
                    assert strand == '-'
                    position = int(end)
                    contig.seq = contig.seq[position:] + contig.seq[:position]
                    contig.revcomp()
                    print(log_prefix, contig.id, position + 1, 'prodigal', 'yes', '-', '-', sep='\t', file=log_fh)
            elif contig.id in to_ignore:
                print(log_prefix, contig.id, '-', '-', '-', '-', 'skipped', sep='\t', file=log_fh)
            else:
                print(log_prefix, contig.id, '-', '-', '-', '-', '-', sep='\t', file=log_fh)

            assert original_length == len(contig)

        pyfastaq.utils.close(log_fh)


    def run(self):
        renamed_contigs, contig_rename_dict = StartFixer._rename_contigs(self.input_assembly)
        if self.verbose:
            print('[fixstart] loaded input contigs', flush=True)

        end_extend = StartFixer._max_length_from_fasta_file(self.genes_fa)
        log_fh = pyfastaq.utils.open_file_write(self.outprefix + '.detailed.log')

        if self.verbose:
            print('[fixstart] Running promer to look for reference genes', flush=True)
        circ_with_promer = StartFixer._find_circular_using_promer(
            self.outprefix + '.promer',
            self.genes_fa,
            renamed_contigs,
            self.min_percent_identity,
            end_extend,
            log_fh,
            ignore=self.ignore,
            promer_mincluster=self.promer_mincluster,
        )

        if self.verbose:
            print('[fixstart] Running prodigal on sequences that had no promer match', flush=True)
        circ_with_prodigal = StartFixer._find_circular_using_prodigal(
            self.outprefix + '.prodigal',
            renamed_contigs,
            circ_with_promer,
            log_fh,
            ignore=self.ignore)

        if self.verbose:
            print('[fixstart] Fixing start positions of contigs', flush=True)
        StartFixer._rearrange_contigs(
            renamed_contigs,
            circ_with_promer,
            circ_with_prodigal,
            self.ignore,
            end_extend,
            self.outprefix + '.log'
        )

        pyfastaq.utils.close(log_fh)

        if self.verbose:
            print('[fixstart] Writing final FASTA file of contigs', flush=True)
        StartFixer._write_renamed_contigs(
            renamed_contigs,
            contig_rename_dict,
            self.outprefix + '.fasta'
        )



import argparse
import os
import sys
import pyfastaq
import circlator

class Error (Exception): pass


def print_message(m, opts):
    if opts.verbose:
        print(m)


def run():
    parser = argparse.ArgumentParser(
        description = 'Run mapreads, bam2reads, assemble, merge, clean, fixstart',
        usage = 'circlator all [options] <assembly.fasta> <reads.fasta/q> <output directory>')
    parser.add_argument('--threads', type=int, help='Number of threads [%(default)s]', default=1, metavar='INT')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('--unchanged_code', type=int, help='Code to return when the input assembly is not changed [%(default)s]', default=0, metavar='INT')
    parser.add_argument('--assembler', choices=circlator.common.allowed_assemblers, help='Assembler to use for reassemblies [%(default)s]', default='spades')
    parser.add_argument('--split_all_reads', action='store_true', help='By default, reads mapped to shorter contigs are left unchanged. This option splits them into two, broken at the middle of the contig to try to force circularization. May help if the assembler does not detect circular contigs (eg canu)')
    parser.add_argument('--data_type', choices=circlator.common.allowed_data_types, help='String representing one of the 4 type of data analysed (only used for Canu) [%(default)s]', default='pacbio-corrected')
    parser.add_argument('assembly', help='Name of original assembly', metavar='assembly.fasta')
    parser.add_argument('reads', help='Name of corrected reads FASTA or FASTQ file', metavar='reads.fasta/q')
    parser.add_argument('outdir', help='Name of output directory (must not already exist)', metavar='output directory')

    mapreads_group = parser.add_argument_group('mapreads options')
    mapreads_group.add_argument('--bwa_opts', help='BWA options, in quotes [%(default)s]', default='-x pacbio', metavar='STRING')

    bam2reads_group = parser.add_argument_group('bam2reads options')
    bam2reads_group.add_argument('--b2r_discard_unmapped', action='store_true', help='Use this to not keep unmapped reads')
    bam2reads_group.add_argument('--b2r_only_contigs', help='File of contig names (one per line). Only reads that map to these contigs are kept (and unmapped reads, unless --b2r_discard_unmapped is used). Note: the whole assembly is still used as a reference when mapping', metavar='FILENAME')
    bam2reads_group.add_argument('--b2r_length_cutoff', type=int, help='All reads mapped to contigs shorter than this will be kept [%(default)s]', default=100000, metavar='INT')
    bam2reads_group.add_argument('--b2r_min_read_length', type=int, help='Minimum length of read to output [%(default)s]', default=250, metavar='INT')

    assemble_group = parser.add_argument_group('assemble options')
    parser.add_argument('--assemble_spades_k', help='Comma separated list of kmers to use when running SPAdes. Max kmer is 127 and each kmer should be an odd integer [%(default)s]', default='127,117,107,97,87,77', metavar='k1,k2,k3,...')
    parser.add_argument('--assemble_spades_use_first', action='store_true', help='Use the first successful SPAdes assembly. Default is to try all kmers and use the assembly with the largest N50')
    parser.add_argument('--assemble_not_careful', action='store_true', help='Do not use the --careful option with SPAdes (used by default)')
    parser.add_argument('--assemble_not_only_assembler', action='store_true', help='Do not use the --assemble-only option with SPAdes (used by default). Important: with this option, the input reads must be in FASTQ format, otherwise SPAdes will crash because it needs quality scores to correct the reads.')

    merge_group = parser.add_argument_group('merge options')
    merge_group.add_argument('--merge_diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    merge_group.add_argument('--merge_min_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    merge_group.add_argument('--merge_min_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    merge_group.add_argument('--merge_min_length_merge', type=int, help='Minimum length of nucmer hit to use when merging [%(default)s]', metavar='INT', default=4000)
    merge_group.add_argument('--merge_min_spades_circ_pc', type=float, help='Min percent of contigs needed to be covered by nucmer hits to spades circular contigs [%(default)s]', metavar='FLOAT', default=95)
    merge_group.add_argument('--merge_breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)
    merge_group.add_argument('--merge_ref_end', type=int, help='max distance allowed between nucmer hit and end of input assembly contig [%(default)s]', metavar='INT', default=15000)
    merge_group.add_argument('--merge_reassemble_end', type=int, help='max distance allowed between nucmer hit and end of reassembly contig [%(default)s]', metavar='INT', default=1000)
    merge_group.add_argument('--no_pair_merge', action='store_true', help='Do not merge pairs of contigs when running merge task')

    clean_group = parser.add_argument_group('clean options')
    clean_group.add_argument('--clean_min_contig_length', type=int, help='Contigs shorter than this are discarded (unless specified using --keep) [%(default)s]', default=2000, metavar='INT')
    clean_group.add_argument('--clean_min_contig_percent', type=int, help='If length of nucmer hit is at least this percentage of length of contig, then contig is removed. (unless specified using --keep) [%(default)s]', default=95, metavar='FLOAT')
    clean_group.add_argument('--clean_diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    clean_group.add_argument('--clean_min_nucmer_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    clean_group.add_argument('--clean_min_nucmer_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    clean_group.add_argument('--clean_breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)

    fixstart_group = parser.add_argument_group('fixstart options')
    fixstart_group.add_argument('--genes_fa', help='FASTA file of genes to search for to use as start point. If this option is not used, a built-in set of dnaA genes is used', metavar='FILENAME')
    fixstart_group.add_argument('--fixstart_mincluster', type=int, help='The -c|mincluster option of promer. If this option is used, it overrides promer\'s default value', metavar='INT')
    fixstart_group.add_argument('--fixstart_min_id', type=float, help='Minimum percent identity of promer match between contigs and gene(s) to use as start point [%(default)s]', default=70, metavar='FLOAT')

    options = parser.parse_args()

    print_message('{:_^79}'.format(' Checking external programs '), options)
    if options.verbose:
        circlator.versions.get_all_versions(sys.stdout, raise_error=True, assembler=options.assembler)
    else:
        circlator.versions.get_all_versions(None, raise_error=True, assembler=options.assembler)


    files_to_check = [options.assembly, options.reads]
    if options.b2r_only_contigs:
        files_to_check.append(options.b2r_only_contigs)
        options.b2r_only_contigs = os.path.abspath(options.b2r_only_contigs)

    if options.genes_fa:
        files_to_check.append(options.genes_fa)

    circlator.common.check_files_exist(files_to_check)

    if options.genes_fa:
        options.genes_fa = os.path.abspath(options.genes_fa)

    original_assembly = os.path.abspath(options.assembly)
    original_reads = os.path.abspath(options.reads)


    try:
        os.mkdir(options.outdir)
    except:
        print('Error making output directory', options.outdir, file=sys.stderr)
        sys.exit(1)

    os.chdir(options.outdir)

    with open('00.info.txt', 'w') as f:
        print(sys.argv[0], 'all', ' '.join(sys.argv[1:]), file=f)
        circlator.versions.get_all_versions(f)

    original_assembly_renamed = '00.input_assembly.fasta'
    bam = '01.mapreads.bam'
    filtered_reads_prefix = '02.bam2reads'
    filtered_reads =  filtered_reads_prefix + ('.fastq' if options.assemble_not_only_assembler else '.fasta')
    assembly_dir = '03.assemble'
    reassembly = os.path.join(assembly_dir, 'contigs.fasta')
    merge_prefix = '04.merge'
    merged_fasta = merge_prefix + '.fasta'
    clean_prefix = '05.clean'
    clean_fasta = clean_prefix + '.fasta'
    fixstart_prefix = '06.fixstart'
    fixstart_fasta = fixstart_prefix + '.fasta'

    pyfastaq.tasks.to_fasta(
        original_assembly,
        original_assembly_renamed,
        strip_after_first_whitespace=True,
        check_unique=True
    )

    #-------------------------------- mapreads -------------------------------
    print_message('{:_^79}'.format(' Running mapreads '), options)
    circlator.mapping.bwa_mem(
      original_assembly_renamed,
      original_reads,
      bam,
      threads=options.threads,
      bwa_options=options.bwa_opts,
      verbose=options.verbose,
    )


    #-------------------------------- bam2reads ------------------------------
    print_message('{:_^79}'.format(' Running bam2reads '), options)
    bam_filter = circlator.bamfilter.BamFilter(
        bam,
        filtered_reads_prefix,
        fastq_out=options.assemble_not_only_assembler,
        length_cutoff=options.b2r_length_cutoff,
        min_read_length=options.b2r_min_read_length,
        contigs_to_use=options.b2r_only_contigs,
        discard_unmapped=options.b2r_discard_unmapped,
        verbose=options.verbose,
        split_all_reads=options.split_all_reads,
    )
    bam_filter.run()


    #-------------------------------- assemble -------------------------------
    print_message('{:_^79}'.format(' Running assemble '), options)
    a = circlator.assemble.Assembler(
        filtered_reads,
        assembly_dir,
        threads=options.threads,
        careful=not options.assemble_not_careful,
        only_assembler=not options.assemble_not_only_assembler,
        spades_kmers=options.assemble_spades_k,
        spades_use_first_success=options.assemble_spades_use_first,
        assembler=options.assembler,
        genomeSize=options.b2r_length_cutoff,
        data_type=options.data_type,
        verbose=options.verbose
    )
    a.run()


    #------------------------------ filter original assembly -----------------
    if options.b2r_only_contigs:
        print_message('{:_^79}'.format(' --b2r_only_contigs used - filering contigs '), options)
        assembly_to_use = merge_prefix + '.00.filtered_assembly.fa'
        pyfastaq.tasks.filter(original_assembly_renamed, assembly_to_use, ids_file=options.b2r_only_contigs)
    else:
        assembly_to_use = original_assembly_renamed


    #-------------------------------- merge ----------------------------------
    print_message('{:_^79}'.format(' Running merge '), options)
    if not options.no_pair_merge:
        merge_reads = filtered_reads
    else:
        merge_reads = None

    m = circlator.merge.Merger(
        assembly_to_use,
        assembly_dir,
        merge_prefix,
        nucmer_diagdiff=options.merge_diagdiff,
        nucmer_min_id=options.merge_min_id,
        nucmer_min_length=options.merge_min_length,
        nucmer_min_length_for_merges=options.merge_min_length_merge,
        min_spades_circular_percent=options.merge_min_spades_circ_pc,
        spades_kmers=options.assemble_spades_k,
        spades_use_first_success=options.assemble_spades_use_first,
        spades_careful=not options.assemble_not_careful,
        spades_only_assembler=not options.assemble_not_only_assembler,
        assembler=options.assembler,
        length_cutoff=options.b2r_length_cutoff,
        split_all_reads=options.split_all_reads,
        data_type=options.data_type,
        nucmer_breaklen=options.merge_breaklen,
        ref_end_tolerance=options.merge_ref_end,
        qry_end_tolerance=options.merge_reassemble_end,
        threads=options.threads,
        verbose=options.verbose,
        reads=merge_reads
    )
    m.run()


    #-------------------------------- clean ----------------------------------
    merge_log = merge_prefix + '.circularise.log'
    contigs_to_keep = []
    contigs_to_not_fix_start = []
    with open(merge_log) as f:
        for line in f:
            if not line.startswith('[merge circularised]\t'):
                continue
            if line.rstrip() == '\t'.join(['[merge circularised]', '#Contig', 'repetitive_deleted', 'circl_using_nucmer', 'circl_using_spades', 'circularised']):
                continue

            x, name, u, y, z, circularised = line.rstrip().split('\t')
            if circularised == '1':
                contigs_to_keep.append(name)
            else:
                contigs_to_not_fix_start.append(name)

    clean_keep_file = clean_prefix + '.contigs_to_keep'
    with open(clean_keep_file, 'w') as f:
        if len(contigs_to_keep) > 0:
            print('\n'.join(contigs_to_keep), file=f)

    not_fix_start_file = fixstart_prefix + '.contigs_to_not_change'
    with open(not_fix_start_file, 'w') as f:
        if len(contigs_to_not_fix_start) > 0:
            print('\n'.join(contigs_to_not_fix_start), file=f)

    print_message('{:_^79}'.format(' Running clean '), options)

    cleaner = circlator.clean.Cleaner(
        merged_fasta,
        clean_prefix,
        min_contig_length=options.clean_min_contig_length,
        min_contig_percent_match=options.clean_min_contig_percent,
        nucmer_diagdiff=options.clean_diagdiff,
        nucmer_min_id=options.clean_min_nucmer_id,
        nucmer_min_length=options.clean_min_nucmer_length,
        nucmer_breaklen=options.clean_breaklen,
        keepfile=clean_keep_file,
        verbose=options.verbose
    )
    cleaner.run()


    #-------------------------------- fixstart -------------------------------
    print_message('{:_^79}'.format(' Running fixstart '), options)
    fixer = circlator.start_fixer.StartFixer(
        clean_fasta,
        fixstart_prefix,
        min_percent_identity=options.fixstart_min_id,
        promer_mincluster=options.fixstart_mincluster,
        genes_fa=options.genes_fa,
        ignore=not_fix_start_file,
        verbose=options.verbose
    )
    fixer.run()

    #-------------------------------- summary -------------------------------
    print_message('{:_^79}'.format(' Summary '), options)
    number_of_input_contigs = pyfastaq.tasks.count_sequences(original_assembly_renamed)
    final_number_of_contigs = pyfastaq.tasks.count_sequences(fixstart_fasta)
    number_circularized = len(contigs_to_keep)
    print_message('Number of input contigs: ' + str(number_of_input_contigs), options)
    print_message('Number of contigs after merging: ' + str(final_number_of_contigs), options)
    print_message(' '.join(['Circularized', str(number_circularized), 'of', str(final_number_of_contigs), 'contig(s)']), options)

    with open(fixstart_prefix + '.ALL_FINISHED', 'w') as f:
        pass

    if number_of_input_contigs == final_number_of_contigs and number_circularized == 0:
        sys.exit(options.unchanged_code)




__all__ = [
    'all',
    'assemble',
    'bam2reads',
    'clean',
    'fixstart',
    'get_dnaa',
    'mapreads',
    'merge',
    'minimus2',
    'progcheck',
    'version',
]

from circlator.tasks import *
